
;***************************
;	FILENAME:	5930_SWEEPER_2or3_INPUTS.ASM
;	DATE:	DECEMBER 21, 2010
;	AUTHOR:	LARRY FOLTZER ):@)
;
;***************************
;	THIS CODE IS USED TO IMPLEMENT A SWEEP-FREQUENCY GENERATOR
;	USING ANALOG DEVICES AD5930 DDS IC
;
;	OPERATION CONTROLLED BY TW0 OR THREE INPUT PARAMETERS.
;
;	1) SWEEP CENTER FREQUENCY, FC
;	2) FREQUENCY INCREMENT, DELTAF
;	3) MULTIPLIER FOR TIME SPENT AT EACH FREQUENCY
;		INITIAL VALUES DEPEND ON CENTER FREQUENCY INPUT
;
;	NUMBER OF FREQUENCY INCREMENTS FIXED AT 500/SWEEP
;
;***************************
;	FILES REQUIRED: P16F627A.INC
;	DATA SHEET:	AD5930 DDS
;
;***************************
;	MCU CONFIGURATION
	LIST      P=16F627A           ; LIST DIRECTIVE TO DEFINE PROCESSOR
	#INCLUDE <P16F627A.INC>       ; PROCESSOR SPECIFIC VARIABLE DEFINITIONS
	ERRORLEVEL  -302              ; SUPPRESS MESSAGE 302 FROM LIST FILE
	__CONFIG   _CP_OFF & _DATA_CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTOSC_OSC_NOCLKOUT
;
;***************************
;	HARDWARE DEFINITIONS FOR MCU 16F627A
;		DATA DIRECTION REGISTER PORT DEFINITIONS
;			TRISA	=	0110 0001	=	0X61
;			TRISB	=	0011 0000	=	0X30
;		INITIAL PORT OUTPUT VALUES
;			PORTA	=	1--0 000-	;	0X80
;			PORTB	=	01-- 0000	;	0X40
;
;***************************
;		AD5310 10-BIT D/A SIGNALS AND PARAMETERS
;		SYNC DRIVEN BY PORTA,7		INIT OUTPUT LEVEL TO 1, DDR TO 0
;		CLK DRIVEN BY PORTB,6		INIT OUTPUT LEVEL TO 1, DDR TO 0
;		DIN DRIVEN BY PORTB,7		INIT OUTPUT LEVEL TO 0, DDR TO 0
;
;***************************
;
;		10-BIT AD5611 D/A SIGNALS AND PIN-OUTS
;		PCB DESIGNED FOR AD5310, MUST FLIP AD5611 UPSIDE DOWN, PIN 6 FOR PIN 1, FOR SIGNAL COMPATIBILITY
;		PIN 4, VDD		PIN 5, GND		PIN 6, VOUT
;		    -		    -		TO OUTPUT BNC
;
;		PIN 3, DATA		PIN 2, CLK		PIN 1, /SYNC
;		MCU PORTB,7		MCU PORTB,6		MCU PORTA,7
;
;***************************
;	AD5930 TIMING INFORMATION AND EXAMPLE
;
;	MAX AD5930 CLOCK FREQUENCY IS 50MHZ, AND HAS AN 24-BIT CORE.
;	WILL OPERATE AD5930 AT 2*(2^24)HZ = 33.554432MHZ = 1/29.8NS
;
;	TO GENERATE 1MHZ OUTPUT (PERIOD=1US) MEANS WE MUST TRAVERSE
;	PHASE-AMPLITUDE TABLE IN (1US)/(29.8NS), I.E., 33.554 STEPS OF SIZE
;	(2^24)/33.554 = 500,000. THEREFORE THE PHASE STEP INCREMENT
;	FOR THE DESIRED OUTPUT FREQUENCY, AND THE "STEP-SIZE" FOR
;	PRODUCTION OF A PARTICULAR FREQUENCY IS HALF THE DESIRED FREQUENCY VALUE.
;
;	E.G., TO PRODUCE A CENTERF = 4.5MHZ (0X44AA20) PHASE STEP SIZE MUST BE = 0X225510
;	WITH DELTAF = 2000HZ = 0X07D0	(PRODUCES 1MHZ TOTAL SWEEP WIDTH, FROM 5MHZ TO 4MHZ)
;	START FREQ PARAMETER IS {FCENTER + 250*DELTAF}	= {0X44AA20 + 0XFA*0X07D0}/2
;					= {0X44AA20 + 0X07A120}/2
;					= 0X4C4B40/2 = 0X2625A0
;
;***************************
;	MULTI-PAGE RAM SPACE ALLOCATION
;	0X070 THRU 0X07F IN SUPPORT OF INDIRECT ADDRESSING IN ANY PAGE

W_TEMP		EQU	0X7E	; VARIABLE USED FOR CONTEXT SAVING 
STATUS_TEMP		EQU	0X7F	; VARIABLE USED FOR CONTEXT SAVING

;***************************
		ORG	0X020
CENTERF		RES	3	; SCALED	MSB,XSB,LSB	= CENTERF,CENTERF+1,CENTERF+2

		ORG	0X028
CNTR_0		RES	1
CNTR_1		RES	1
CNTR_2		RES	1

		ORG	0X030
DELTAF		RES	3	; SCALED	MSB,XSB,LSB

		ORG	0X038
BIT_COUNT		RES	1
UNITS		RES	1	; DEFINES TIME AT FREQUENCY IN STEPS OF TBD

		ORG	0X040
STARTF		RES	3	; SCALED	MSB,XSB,LSB

		ORG	0X048
;	HHD & LHD = HIGH AND LOW BYTES OF HORIZONTAL DRIVE DISPLACEMENT
HHD		RES	1	; BITS (B9:B8) OF 10-BIT D/A DISPLACEMENT
LHD		RES	1	; LOWER 8-BITS (B7:B0) OF D/A DISPLACEMENT

		ORG	0X050
VAL		RES	3
CMMD		RES	1

		ORG	0X058
SUM		RES	3

		ORG	0X060
KEYCODE		RES	1	; HOLDS DECIMAL INPUT
KEY_CNT		RES	1	; NUMBER OF NUMERICAL CHARACTERS IN INPUT

		ORG	0X068
SRB3		RES	1	; 1ST BYTE OF 32-BIT WORD TO BE TRASMITTED
SRB2		RES	1
SRB1		RES	1
SRB0		RES	1

;***************************
;	PORTA BIT ASSIGNMENTS
DAV		EQU	0	; SIGNAL FROM KEYPAD
ACK		EQU	1	; DAV ACKNOWLEDGEMENT TO KEYPAD
D5LED		EQU	2
D4LED		EQU	3
D3LED		EQU	4	; NEEDS PULLUP
VPP		EQU	5
KEYPAD_DATA		EQU	6
D2A_FRAME		EQU	7	; MCU OUTPUT

;	PORTB BIT ASSIGNMENTS			
DDS_FRAME		EQU	0	; MCU OUTPUT, AD5930 USE
DDS_DATA		EQU	1	; MCU OUTPUT, AD5930 USE
DDS_CLK		EQU	2	; MCU OUTPUT, AD5930 USE
SWEEP_START		EQU	3	; MCU OUTPUT, HIGH STARTS SWEEP, USED FOR AD5930 ONLY
IO_1		EQU	4	; DDS MSB OUTPUT MONITOR FOR AD5930
				; PHASE SELECT LINE FOR AD9834 OUTPUT MODULATION
SWEEP_SYNC		EQU	5	; MCU INPUT DETECTS DDS (+) PULSE AT END OF TOTAL SWEEP CYCLE
				; FREQUENCY SELECT LINE FOR AD9834 OUTPUT MODULATION
D2A_CLK		EQU	6	; MCU OUTPUT, AD5930 USE ONLY
D2A_DATA		EQU	7	; MCU OUTPUT, AD5930 USE ONLY

;***************************
	ORG    	 0X000             ; PROCESSOR RESET VECTOR
	GOTO   	 INIT_HW           ; GO TO BEGINNING OF PROGRAM

	ORG    	 0X004             ; INTERRUPT VECTOR LOCATION
	RETFIE
;
;***************************
	ORG	0X010
GET_TIME	;	OVERHEAD OF 99US TO CHANGE HORIZONTAL D2A TAKEN INTO ACCOUNT
	ADDWF		PCL,F
	RETLW		0X78		; FOR 10MS
	RETLW		0X40		; FOR 5MS
	RETLW		0X04		; FOR 500US
	RETLW		0X00		; FOR 50US
	RETLW		0X00		; FOR 50US
	RETLW		0X00		; FOR 50US
	RETLW		0X00		; FOR 50US
	RETLW		0X00		; FOR 50US

;***************************
;	KEYPAD MAP AND LABELING
;	0	1	2	3
;	4	5	6	7
;	8	9	SEQ	RST
;	ENT	DEL	TKO	ABT

;***************************
CMMD_KEY
	ADDWF	PCL,F
	GOTO	INIT_HW		; SEQ
	GOTO	INIT_HW		; RST
	GOTO	CENTER_FREQ		; ENT
	GOTO	FREQ_INCREMENT	; DEL
	GOTO	CYCLES		; TKO
	GOTO	INIT_HW		; ABT

;***************************
	ORG	0X020
;	DO NOT KNOW STATE. WILL STEP THROUGH UNTIL WE GET A SYNC PULSE, AND GO FROM THERE.
;	NO HORIZONTAL DRIVE AT THIS POINT.
TURN_RF_OFF
	BCF	PORTB,SWEEP_START
	CALL	W850US
SYNC_UP
	BSF	PORTB,SWEEP_START
	CALL	W850US
	BTFSS	PORTB,SWEEP_SYNC
	GOTO	TURN_RF_OFF
	BCF	PORTB,SWEEP_START
	GOTO	PREP4SWEEP

;***************************
W850US
	MOVLW	.15
	MOVWF	CNTR_0
W8LP
	DECFSZ	CNTR_0,F
	GOTO	W8LP
	RETURN

;***************************
PREP4SWEEP
	CALL	RESET_HORIZ

	CLRF	SUM
	CLRF	SUM+1
	CLRF	SUM+2
	CLRF	VAL
	CLRF	VAL+1
	CLRF	VAL+2

	CLRF	KEY_CNT

;***************************
IN_SYNC
	BSF	PORTA,D4LED
	BSF	PORTB,SWEEP_START
	NOP
	BTFSS	PORTB,SWEEP_SYNC
	GOTO	DO_CARRIER

;***************************
END_OF_SWEEP
	BCF	PORTB,SWEEP_START
	CALL	RESET_HORIZ
	BTFSC	PORTA,DAV
	GOTO	IN_SYNC	; NO INPUT SO DO ANOTHER SWEEP
	BCF	PORTA,D4LED		; LED OFF TO ACK INPUT
	GOTO	GET_KEYPAD_BYTE

;***************************
DO_CARRIER
	MOVF	UNITS,W
	MOVWF	CNTR_1
	BTFSC	STATUS,Z
	GOTO	OVERHEAD_IS_200US
CARRIER_TIME
	MOVLW	.15	; OR THERE ABOUT, FOR 50US
	MOVWF	CNTR_0
CTLOOP
	DECFSZ	CNTR_0,F
	GOTO	CTLOOP
	DECFSZ	CNTR_1,F
	GOTO	CARRIER_TIME
OVERHEAD_IS_200US
; WE TURN CARRIER OFF AT VERY END OF "MAKE_SEND_D2A_CMMD" TO MAXIMIZE CARRIER ON TIME PERCENTAGE
	CALL	CALC_NEW_XPOS	; 11~
	CALL	MAKE_SEND_D2A_CMMD	; 11~ + 3~ + (15 X 11~) + 12~
	GOTO	IN_SYNC		; = 37~ + 165~ = 202~ ==> ~99US = OVERHEAD

;***************************
GET_KEYPAD_BYTE
	CLRF	KEYCODE
	MOVLW	0X04
	MOVWF	BIT_COUNT
WATCH4BIT
	BTFSC	PORTA,DAV	;	DAV_INPUT
	GOTO	WATCH4BIT
	BSF	STATUS,C
	BTFSS	PORTA,KEYPAD_DATA	; SDATA_INPUT
	BCF	STATUS,C
	RLF	KEYCODE,F
	BCF	PORTA,ACK	;	ACK_OUTPUT
W84NODAV
	BTFSS	PORTA,DAV	;	DAV_INPUT ON PA0
	GOTO	W84NODAV
	BSF	PORTA,ACK
	DECFSZ	BIT_COUNT,F
	GOTO	WATCH4BIT
	MOVLW	0X0A
	SUBWF	KEYCODE,W
	BTFSC	STATUS,C
	GOTO	CMMD_KEY	; => 0X0A
;	GOTO	NUM_KEY

;***************************
NUM_KEY	; INPUT IS POTENTIALLY 24-BITS LONG, NEEDING 3-BYTES STORAGE
;	ROUTINE TO MAKE HEX OF BASE 10 INPUT
;	FOLLOWING CODE DOUBLES CONTENTS OF "VAL" AND SAVES IN "SUM" AND "VAL"
	INCF	KEY_CNT,F
	BCF	STATUS,C
	RLF	SUM+2,W	;	LSB
	MOVWF	SUM+2
	MOVWF	VAL+2
	RLF	SUM+1,W
	MOVWF	SUM+1
	MOVWF	VAL+1
	RLF	SUM,W	;	MSB
	MOVWF	SUM
	MOVWF	VAL

;	FOLLOWING CODE CONTINUES SHIFTING "VAL" TO COMPUTE EIGHT TIMES INPUT VALUE
	BCF	STATUS,C
	RLF	VAL+2,F
	RLF	VAL+1,F
	RLF	VAL,F	;	4X
	BCF	STATUS,C
	RLF	VAL+2,F
	RLF	VAL+1,F
	RLF	VAL,F	;	8X

;	ADD 2X (STORED IN "SUM") TO 8X (STORED IN "VAL"), LEAVING RESULT IN "SUM"
	MOVF	VAL+2,W
	ADDWF	SUM+2,F
	MOVF	VAL+1,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	SUM+1,F
	MOVF	VAL,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	SUM,F

;	ADD KEYCODE TO SUM
	MOVF	KEYCODE,W
	ADDWF	SUM+2,F
	BTFSS	STATUS,C
	GOTO	GET_KEYPAD_BYTE
	MOVLW	.1
	ADDWF	SUM+1,F
	BTFSS	STATUS,C
	GOTO	GET_KEYPAD_BYTE
	ADDWF	SUM,F
	GOTO	GET_KEYPAD_BYTE

;***************************
MAKE_SEND_D2A_CMMD
;	CONVERT D2A OFFSET TO COMMAND FORMAT
	SWAPF	HHD,W
	ANDLW	0X30
	MOVWF	SRB1
	SWAPF	LHD,W
	MOVWF	SRB0
	ANDLW	0X0F
	IORWF	SRB1,F
	MOVLW	0XF0
	ANDWF	SRB0,F
;
SEND15_16	
	MOVLW	.15
	MOVWF	CNTR_2
	BCF	PORTA,D2A_FRAME
SEND2D2ALOOP
	RLF	SRB0,F
	RLF	SRB1,F
	BTFSS	STATUS,C
	BCF	PORTB,D2A_DATA
	BTFSC	STATUS,C
	BSF	PORTB,D2A_DATA
	BCF	PORTB,D2A_CLK
	BSF	PORTB,D2A_CLK
	DECFSZ	CNTR_2,F
	GOTO	SEND2D2ALOOP
; MOVE LAST BIT TO OUTPUT
	RLF	SRB0,F
	RLF	SRB1,F
	BTFSS	STATUS,C
	BCF	PORTB,D2A_DATA
	BTFSC	STATUS,C
	BSF	PORTB,D2A_DATA
; WE TURN OFF CARRIER HERE BECAUSE OF LONG, 200US, DELAY IN UPDATING HORIZ D2A
	BCF	PORTB,SWEEP_START
; THEN WE TRIGGER D2A UPDATE AND "RETURN TO SENDER" ELVIS WOULD HAVE LIKED THIS APPROACH
	BCF	PORTB,D2A_CLK
	BSF	PORTB,D2A_CLK
	BSF	PORTA,D2A_FRAME	; PORTA,7
	RETURN

;***************************
CALC_NEW_XPOS
;	DECREMENT D2A BY 2, ADD SIGN-EXTENDED
;	TWOS-COMPLEMENT OF 2 = 0XFFFE TO CURRENT
;	VALUE AND REPEAT FOR EACH FREQUENCY STEP
;	IT WILL MOVE_SWEEP LEFT 1 SPACE
	MOVLW	0XFE
	ADDWF	LHD,F
	MOVF	HHD,W
	BTFSC	STATUS,C
	INCF	HHD,W
	ADDLW	0XFF
	MOVWF	HHD
	RETURN

;***************************
RESET_HORIZ
	MOVLW	0X03
	MOVWF	HHD	; HIGH-BYTE OF HORIZONTAL DEFLECTION
	MOVLW	0XF4
	MOVWF	LHD	; LOW-BYTE OF HORIZONTAL DEFLECTION
	CALL	MAKE_SEND_D2A_CMMD
	RETURN

;***************************
CENTER_FREQ
; SAVE NUM OF 50US UNITS
	MOVLW	.3
	SUBWF	KEY_CNT,W
	CALL	GET_TIME
	MOVWF	UNITS
;
	BCF	STATUS,C
	RRF	SUM,W	; MSB
	MOVWF	CENTERF
	RRF	SUM+1,W
	MOVWF	CENTERF+1
	RRF	SUM+2,W	; LSB
	MOVWF	CENTERF+2
	CALL	COMPUTE_FS
	CALL	ASSEMBLE_STARTF_COMMAND
	CALL	SEND_32_DDS_BITS
	GOTO	PREP4SWEEP

;***************************
COMPUTE_FS	;	SCALED FOR DDS REGISTER PARAMETER
;
;	MSB			LSB
;	CF		CF+1	CF+2
;	DF		DF+1	DF+2
;	SF		SF+1	SF+2
;
;***************************
;	SHIFT DELTAF LEFT 8-BITS, MULTIPLY BY 256,
;	AND SAVE 256*DELTAF IN STARTF.
;	ALSO SAVE 1*DELTAF IN VAL FOR LATER WORK.
	CLRF	STARTF+2
	MOVF	DELTAF+2,W
	MOVWF	VAL+2
	MOVWF	STARTF+1
	MOVF	DELTAF+1,W
	MOVWF	VAL+1
	MOVWF	STARTF
	MOVF	DELTAF,W
	MOVWF	VAL

;***************************
;	NOW MULTIPLY VAL (HOLDS DELTAF) BY TWO (2) AND SAVE IN VAL & SUM.
	BCF	STATUS,C
	RLF	VAL+2,W
	MOVWF	VAL+2
	MOVWF	SUM+2
	RLF	VAL+1,W
	MOVWF	VAL+1
	MOVWF	SUM+1
	RLF	VAL,W
	MOVWF	VAL
	MOVWF	SUM

;***************************
;	DOUBLE SUM TO MAKE 4*DELTAF
	BCF	STATUS,C
	RLF	SUM+2,F
	RLF	SUM+1,F
	RLF	SUM,F

;***************************
;	ADD VAL (2*DELTAF) TO SUM (4*DELTAF), MAKING 6*DELTAF WITH RESULT IN SUM.
	MOVF	VAL+2,W
	ADDWF	SUM+2,F
	MOVF	VAL+1,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	SUM+1,F
	MOVF	VAL,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	SUM,F

;***************************
;	NOW TIME TO NEGATE (COMPLEMENT + 1) SUM
	COMF	SUM+2,F
	COMF	SUM+1,F
	COMF	SUM,F
	MOVLW	.1
	ADDWF	SUM+2,F
	MOVF	SUM+1,W
	BTFSC	STATUS,C
	ADDLW	.1
	MOVWF	SUM+1
	MOVF	SUM,W
	BTFSC	STATUS,C
	ADDLW	.1
	MOVWF	SUM

;***************************
;	ADD NEGATED SUM OF 6*DELTAF TO 256*DELTAF HELD IN STARTF
	MOVF	SUM+2,W
	ADDWF	STARTF+2,F
	MOVF	SUM+1,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	STARTF+1,F
	MOVF	SUM,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	STARTF,F

;***************************
;	FINAL STEP: ADD CENTER FREQUENCY TO OBTAIN STARTF
	MOVF	CENTERF+2,W
	ADDWF	STARTF+2,F
	MOVF	CENTERF+1,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	STARTF+1,F
	MOVF	CENTERF,W
	BTFSC	STATUS,C
	ADDLW	.1
	ADDWF	STARTF,F
	RETURN

;***************************
ASSEMBLE_STARTF_COMMAND
;	ADD ADDRESS BITS TO WORDS
	MOVLW	0X0F
	ANDWF	STARTF+1,W
	IORLW	0XC0
	MOVWF	SRB3
	MOVF	STARTF+2,W
	MOVWF	SRB2
	SWAPF	STARTF,W
	ANDLW	0X0F
	IORLW	0XD0
	MOVWF	SRB1
	SWAPF	STARTF+1,W
	ANDLW	0X0F
	MOVWF	SRB0
	SWAPF	STARTF,W
	ANDLW	0XF0
	IORWF	SRB0,F
	RETURN

;***************************
ASSEMBLE_DELTAF_COMMAND
;	ADD ADDRESS BITS TO WORDS
	MOVLW	0X0F
	ANDWF	DELTAF+1,W
	IORLW	0X20
	MOVWF	SRB3
	MOVF	DELTAF+2,W
	MOVWF	SRB2
	SWAPF	DELTAF,W
	ANDLW	0X0F
	IORLW	0X38
	MOVWF	SRB1
	SWAPF	DELTAF+1,W
	ANDLW	0X0F
	MOVWF	SRB0
	SWAPF	DELTAF,W
	ANDLW	0XF0
	IORWF	SRB0,F
	RETURN

;***************************
SEND_32_DDS_BITS
	BCF	PORTB,DDS_FRAME
	MOVLW	.8
	MOVWF	BIT_COUNT
SEND_SRB3
	RLF	SRB3,F
	BTFSC	STATUS,C
	BSF	PORTB,DDS_DATA
	BTFSS	STATUS,C
	BCF	PORTB,DDS_DATA
	BCF	PORTB,DDS_CLK
	BSF	PORTB,DDS_CLK
	DECFSZ	BIT_COUNT,F
	GOTO	SEND_SRB3
	MOVLW	.8
	MOVWF	BIT_COUNT
SEND_SRB2
	RLF	SRB2,F
	BTFSC	STATUS,C
	BSF	PORTB,DDS_DATA
	BTFSS	STATUS,C
	BCF	PORTB,DDS_DATA
	BCF	PORTB,DDS_CLK
	BSF	PORTB,DDS_CLK
	DECFSZ	BIT_COUNT,F
	GOTO	SEND_SRB2
SEND_DDS_WORD
	BCF	PORTB,DDS_FRAME
	MOVLW	.8
	MOVWF	BIT_COUNT
SEND_SRB1
	RLF	SRB1,F
	BTFSC	STATUS,C
	BSF	PORTB,DDS_DATA
	BTFSS	STATUS,C
	BCF	PORTB,DDS_DATA
	BCF	PORTB,DDS_CLK
	BSF	PORTB,DDS_CLK
	DECFSZ	BIT_COUNT,F
	GOTO	SEND_SRB1
	MOVLW	.8
	MOVWF	BIT_COUNT
SEND_SRB0
	RLF	SRB0,F
	BTFSC	STATUS,C
	BSF	PORTB,DDS_DATA
	BTFSS	STATUS,C
	BCF	PORTB,DDS_DATA
	BCF	PORTB,DDS_CLK
	BSF	PORTB,DDS_CLK
	DECFSZ	BIT_COUNT,F
	GOTO	SEND_SRB0
	BSF	PORTB,DDS_FRAME
	RETURN

;***************************
FREQ_INCREMENT
;	DEVIDE BY 2 AND COPY TO DELTAF
	BCF	STATUS,C
	RRF	SUM,W	;	MSB
	MOVWF	DELTAF
	RRF	SUM+1,W
	MOVWF	DELTAF+1
	RRF	SUM+2,W	;	LSB
	MOVWF	DELTAF+2
	CALL	COMPUTE_FS
	CALL	ASSEMBLE_STARTF_COMMAND
	CALL	SEND_32_DDS_BITS
	CALL	ASSEMBLE_DELTAF_COMMAND
	CALL	SEND_32_DDS_BITS
	GOTO	PREP4SWEEP

;***************************
CYCLES
;	TIMING INFO:	ASSUME 1 UNIT TIME INTERVAL = 20US
;
;	AT LOWEST FREQUENCY (~= 1000 HZ),
;	FREQUENCY DURATION IS 250 X 20US = 1/200HZ
;	SWEEP DURATION = 500 X 5.0MS = 2.5 SECONDS
;
;	AT 10MHZ, CYCLE COUNT IS 20US X 10MHZ = 200
;	SWEEP DURATION = 500 X 20US = 10MS = 100 SWEEPS/SEC
	MOVF	SUM+2,W
	MOVWF	UNITS
	GOTO	PREP4SWEEP

;***************************
INIT_HW
	MOVLW	0X07
	MOVWF	CMCON
	MOVLW	0X82
	MOVWF	PORTA
	MOVLW	0X65		; 0110 0101
	MOVWF	PORTB

	BSF	STATUS,RP0
	CLRF	OPTION_REG
	MOVLW	0X61
	MOVWF	PORTA
	MOVLW	0X30		; 0011 0000
	MOVWF	PORTB
	BCF	STATUS,RP0

;***************************
; THE FOLLOWING VALUES PRODUCE A SWEEP THAT SPANS
; SPANS 2.0MHZ  TO  8.0MHZ
; AT A LATER DATE I WILL ADD FEATURE TO STORE
; A FREQUENCY PROFILE IN EEPROM FOR RETRIEVAL
; AND STARTUP DEFAULT.
;***************************
CREG_INIT
	MOVLW	0X0E		; NO MSB OUTPUT
	MOVWF	SRB1
	MOVLW	0X7F
	MOVWF	SRB0
	BCF	PORTB,DDS_FRAME
	CALL	SEND_DDS_WORD

;***************************
;	CENTERF = 5MHZ = 2 X 0X2625A0
	MOVLW	0X26
	MOVWF	CENTERF
	MOVLW	0X25
	MOVWF	CENTERF+1
	MOVLW	0XA0
	MOVWF	CENTERF+2

;***************************
;	DELTAF = 12KHZ = 2 X 0X1770
	MOVLW	0X00
	MOVWF	DELTAF
	MOVLW	0X17
	MOVWF	DELTAF+1		
	MOVLW	0X70
	MOVWF	DELTAF+2

;***************************
	CALL	COMPUTE_FS
	CALL	ASSEMBLE_STARTF_COMMAND
	CALL	SEND_32_DDS_BITS

;***************************
; SEND DELTAF PARAMETER
	CALL	ASSEMBLE_DELTAF_COMMAND
	CALL	SEND_32_DDS_BITS

;***************************
;	STEPS INIT;	ONLY DOES 500 STEPS (+-250 ABOUT CENTERF) PER SWEEP
	MOVLW	0X11
	MOVWF	SRB1
	MOVLW	0XF4
	MOVWF	SRB0
	BCF	PORTB,DDS_FRAME
	CALL	SEND_DDS_WORD

;***************************
; FREQUENCY DWELL TIME IN UNITS OF 50US
	MOVLW	0X00	; .250 FOR ~0.1 SECONDS PER FREQUENCY
	MOVWF	UNITS
	GOTO	TURN_RF_OFF

;***************************
	END

DS_BITS

;***************************
; SEND DELTAF PARAMETER
	CALL	ASSEMBLE_DELTAF_COMMAND
	CALL	SEND_32_DDS_BITS

;***************************
;	STEPS INIT;	ONLY DOES 500 STEPS (+-250 ABOUT CENTERF) PER SWEEP
	MOVLW	0X11
	MOVWF	SRB1
	MOVLW	0XF4
	MOVWF	SRB0
	BCF	PORTB,DDS_FRAME
	CALL	SEND_DDS_WORD

;***************************
; FREQUENCY DWELL TIME IN UNITS OF 50US
	MOVLW	0X00	; .250 FOR ~0.1 SECONDS PER 