;**********************************************************************
;
; CPU EE memory I/O
;
;**********************************************************************

;----------------------------------------------------------------------
; Read a "block" (up to 256 bytes) from EE
;----------------------------------------------------------------------

EE_readblock ; Sent EEADR = source, FSR0 = destination, WREG = bytes
             ; Returns Z if chksum matches
             ; ALWAYS destroys the destination buffer

    movwf   ee_count                ; save transfer count
    clrf    ee_chksum               ; init chksum

    clrf            EEADRH          ; prepare to access EE (vs. Flash)
    bcf     EECON1, EEPGD
    bcf     EECON1, CFGS 

    ; 1'st word of block must be the signature (constant)
    rcall   EE_readblock_byte
    xorlw   high(EE_SIGNATURE)
    bnz     EE_readblock_exit
    rcall   EE_readblock_byte
    xorlw   low(EE_SIGNATURE)
    bnz     EE_readblock_exit

    ; read data (although it might not be valid)
EE_readblock_loop
    rcall   EE_readblock_byte
    movwf   POSTINC0
    decfsz  ee_count,f
    bra     EE_readblock_loop

    ; last byte must be the [xor] chksum
    rcall   EE_readblock_byte
EE_readblock_exit
    return


EE_readblock_byte
    bsf     EECON1, RD              ; perform EE read
    movf    EEDATA, w
    incf    EEADR, f
    xorwf   ee_chksum,f
    return


;----------------------------------------------------------------------
; Write a "block" to EE (add signature & chksum)
;----------------------------------------------------------------------

EE_storeblock ; Sent EEADR = destination, FSR0 = source, WREG = bytes

; Failsafe: - if something goes wrong, the watchdog will stop EE writes after ~500 bytes (1 sec @ 2ms/byte)

#if ENABLE_EE_WRITES == 1

    movwf   ee_count                ; save transfer count
    clrf    ee_chksum               ; init chksum

    clrf            EEADRH          ; prepare to access EE (vs. Flash)
    bcf     EECON1, EEPGD
    bcf     EECON1, CFGS 
    bsf     EECON1, WREN

    ; 1'st word of block is the signature (constant)
    movlw   high(EE_SIGNATURE)
    rcall   EE_storeblock_byte
    movlw   low(EE_SIGNATURE)
    rcall   EE_storeblock_byte

    ; write data 
EE_storeblock_loop
    movf    POSTINC0, w
    rcall   EE_storeblock_byte
    decfsz  ee_count, f
    bra     EE_storeblock_loop

    ; last byte is the chksum
    movf    ee_chksum, w
    rcall   EE_storeblock_byte

    ; end EE write
    bcf     EECON1, WREN
#endif
    return


EE_storeblock_byte
    movwf   EEDATA
    xorwf   ee_chksum,f
  DISABLE_INTERRUPTS ; low + high
    movlw   0x55                    ; protected EE write sequence
    movwf   EECON2
    movlw   0xAA
    movwf   EECON2
    bsf     EECON1, WR              ;   start EE write ( ~ 2 ms)
  ENABLE_INTERRUPTS  ; low + high
    ; don't touch EEADR until write is done!
EE_storeblock_wait
    btfsc   EECON1, WR              ; wait for EE write to complete
    bra     EE_storeblock_wait
    incf    EEADR, f
    return
