;**********************************************************************
; LCD Hardware Driver
;
; Nokia 6100 Color LCD 130x130 pixels
;
;    All functions in this module may be called from foreground only.
;
; RSP 2008 - tuned for 40 MHz fOsc, plus shared SPI/I2C port with other peripherals at interrupt.
; RSP 2008 - merged headlight V4 to support panasonic LCD
; RSP 2010 - cloned CID variant, cleaned up & added TEXT_ARAY support
;**********************************************************************

#if LCD_CHIP == EPSON
; Epson S1D15G10 Command Set
#define DISON       0xaf
#define DISOFF      0xae
#define DISNOR      0xa6
#define DISINV      0xa7
#define SLPIN       0x95
#define SLPOUT      0x94
#define COMSCN      0xbb
#define DISCTL      0xca
#define PASET       0x75
#define CASET       0x15
#define DATCTL      0xbc
#define RGBSET8     0xce
#define RAMWR       0x5c
#define RAMRD       0x5d
#define PTLIN       0xa8
#define PTLOUT      0xa9
#define RMWIN       0xe0
#define RMWOUT      0xee
#define ASCSET      0xaa
#define SCSTART     0xab
#define OSCON       0xd1
#define OSCOFF      0xd2
#define PWRCTR      0x20
#define VOLCTR      0x81
#define VOLUP       0xd6
#define VOLDOWN     0xd7
#define TMPGRD      0x82
#define EPCTIN      0xcd
#define EPCOUT      0xcc
#define EPMWR       0xfc
#define EPMRD       0xfd
#define EPSRRD1     0x7c
#define EPSRRD2     0x7d
#define LCD_NOP     0x25
#endif

#if LCD_CHIP == PHILIPS
; Philips PCF8833 command set
#define MADCTL      0x36    ; memory addressing control
#define COLMOD      0x3A    ; color mode
#define RGBSET      0x2D    ; color LUT
#define BSTRON      0x03    ; booster on
#define DISPON      0x29    ; display on
#define SLPOUT      0x11    ; sleep out
#define VOLCTR      0x25    ; contrast
#define CASET       0x2A    ; column set
#define PASET       0x2B    ; page set
#define RAMWR       0x2C    ; write RAM
#define LCD_NOP     0x00
#endif


;----------------------------------------------------------------------
; helper macros to talk to the LCD chip
;----------------------------------------------------------------------

LCD_SPI macro       ; set up SPI port for LCD
    movff   LCD_CONST_SSPCON1, SSPCON1
    movff   LCD_CONST_SSPSTAT, SSPSTAT
    endm

LCDCMD macro zCmd   ; send command (constant) to LCD
    movlw   zCmd
    rcall   do_LCD_command
    endm

LCDDATA macro zData ; send data (constant) to LCD
    movlw   zData
    rcall   do_LCD_data
    endm

LCDREG macro zAddr  ; send data (variable) to LCD
    movf    zAddr, w
    rcall   do_LCD_data
    endm

do_LCD_command  ; send command byte to LCD chip

    ; manually control SPI to send out initial clocks
    bcf     SSPCON1, SSPEN      ; make SPI pins digital outputs
    bsf     LCD_SCL
    bcf     LCD_CS
    bcf     LCD_SDA
    bcf     LCD_SCL
    bsf     LCD_SCL

    bra     do_LCD_common

do_LCD_data     ; send data byte to LCD chip

    ; manually control SPI to send out initial clock
    bcf     SSPCON1, SSPEN      ; make SPI pins digital outputs
    bsf     LCD_SCL
    bcf     LCD_CS
    bsf     LCD_SDA
    bcf     LCD_SCL
    bsf     LCD_SCL

do_LCD_common
    ; use SPI to send out the rest of the bits
    bsf     SSPCON1, SSPEN      ; SPI port is now SPI controlled
    SSP_WRITE_REG WREG
    bsf     LCD_CS

    return


;----------------------------------------------------------------------
; power-on initialization of the LCD controller chip
;----------------------------------------------------------------------

LCD_initialize

    bcf     LCD_RESET
    rcall   lcd_init_delay
    bsf     LCD_RESET
    rcall   lcd_init_delay

    LCD_SPI         ; set up SPI port for LCD

#if LCD_CHIP == EPSON
    LCDCMD  DISCTL
    LCDDATA 0       ; 2 divisions, switching period=8 (default)
    LCDDATA 32      ; nlines/4 - 1 = 132/4 - 1 = 32)
    LCDDATA 0       ; no inversely highlighted lines
    LCDDATA 0       ; dispersion

    LCDCMD  COMSCN  ; Common scan directions (internal chip wiring config)
    LCDDATA 1       ; Scan 1->80, 160<-81

    LCDCMD  OSCON   ; turn on internal oscillator

    LCDCMD  SLPOUT  ; Sleep out

    LCDCMD  VOLCTR  ; electronic volume, this is the contrast/brightness
    LCDDATA 12      ; volume (contrast) setting - fine tuning
    LCDDATA 0x03    ; internal resistor ratio - coarse adjustment

    LCDCMD  PWRCTR  ; power ctrl
    LCDDATA 0x0f    ; everything on, no external reference resistors

    LCDCMD  DISINV  ; invert display colors (so that 0x00=black, 0xFF=white)

    rcall   LCD_set_orientation

    LCDCMD  RGBSET8 ; setup 8-bit color lookup table  [RRRGGGBB]
    ;RED
    LCDDATA 0
    LCDDATA 1
    LCDDATA 3
    LCDDATA 5
    LCDDATA 7
    LCDDATA 10
    LCDDATA 12
    LCDDATA 15
    ;GREEN
    LCDDATA 0
    LCDDATA 1
    LCDDATA 3
    LCDDATA 5
    LCDDATA 7
    LCDDATA 10
    LCDDATA 12
    LCDDATA 15
    ;BLUE
    LCDDATA 0
    LCDDATA 4
    LCDDATA 7
    LCDDATA 15

    LCDCMD  DISON   ; // display on

    rcall   LCD_set_contrast
#endif

#if LCD_CHIP == PHILIPS

    LCDCMD SLPOUT   ; wake up

    rcall   LCD_set_orientation

    LCDCMD COLMOD   ; color mode
    LCDDATA 0x03    ;   12 bits/pixel

    rcall   LCD_set_contrast

    LCDCMD DISPON   ; display on

    LCDCMD BSTRON   ; booster on
#endif

    return


lcd_init_delay ; Philips chip actually requires quite a long reset pulse
    CLR     fgnd_temp,2
lcd_init_delay_loop         
    decfsz  fgnd_temp+0,f
    bra     lcd_init_delay_loop ; 0.1us * 3 * 256 => 76.8us
    decfsz  fgnd_temp+1,f
    bra     lcd_init_delay_loop ; 76.8us * 256 = ~20ms
    return


;----------------------------------------------------------------------
; [re]set LCD orientation (normal vs. inverted)
;                DATCTL    X      Y
;   0 = normal     x00   PASET  CASET
;   1 = 90         x06   CASET  PASET    (addressing swapped)
;   2 = inverted   x03   PASET  CASET    (X and Y inverted)
;   3 = 270        x05   CASET  PASET    (addressing swapped)
;----------------------------------------------------------------------

LCD_set_orientation ; from LCD_orientation

    LCD_SPI         ; set up SPI port for LCD

#if LCD_CHIP == EPSON
    ; set LCD chip
    LCDCMD  DATCTL  ; data control
    rcall   get_orient_DATCTL
    rcall   do_LCD_data
    LCDDATA 0x00    ; normal RGB arrangement
    LCDDATA 0x01    ; 8-bit grayscale

    ; normal addressing
    MOVLFF CASET, DISPyaset
    MOVLFF PASET, DISPxaset
    btfss   LCD_orientation, B0
    bra     LCD_set_orient_addr
    ; inverted addressing
    MOVLFF PASET, DISPyaset
    MOVLFF CASET, DISPxaset
LCD_set_orient_addr

    ; set up orientation offsets
    rcall   get_orient_X
    movff   WREG, DISPxOffset
    rcall   get_orient_Y
    movff   WREG, DISPyOffset
    return

get_orient_DATCTL
    ; jump table
    MOVLF   high(orient_DATCTL_table), PCLATH
    rlncf   LCD_orientation, w  ; 2 bytes/instruction
    addlw   low(orient_DATCTL_table)
    btfsc   STATUS, C
    incf    PCLATH, f
    movwf   PCL
orient_DATCTL_table
    retlw   0
    retlw   6
    retlw   3
    retlw   5

get_orient_X
    ; jump table
    MOVLF   high(orient_Xoffset_table), PCLATH
    rlncf   LCD_orientation, w  ; 2 bytes/instruction
    addlw   low(orient_Xoffset_table)
    btfsc   STATUS, C
    incf    PCLATH, f
    movwf   PCL
orient_Xoffset_table
    retlw   2
    retlw   2
    retlw   0
    retlw   0

get_orient_Y
    ; jump table
    MOVLF   high(orient_Yoffset_table), PCLATH
    rlncf   LCD_orientation, w  ; 2 bytes/instruction
    addlw   low(orient_Yoffset_table)
    btfsc   STATUS, C
    incf    PCLATH, f
    movwf   PCL
orient_Yoffset_table
    retlw   0
    retlw   2
    retlw   2
    retlw   0
#endif

#if LCD_CHIP == PHILIPS
;WARNING - this only implements orientation 0
    ; normal addressing
    MOVLFF CASET, DISPyaset
    MOVLFF PASET, DISPxaset
    ; Philips offsets are always == 1 [coordinates system is (0,0)-(129,129)]
    MOVLFF  1, DISPxOffset
    MOVLFF  1, DISPyOffset
    ; set LCD chip
    LCDCMD MADCTL   ; memory addressing control
    movlw   LCD_ADCTL
    bra     do_LCD_data
#endif


;----------------------------------------------------------------------
; [re]set the LCD contrast
;----------------------------------------------------------------------

LCD_set_contrast    ; from LCD_contrast

    LCD_SPI         ; set up SPI port for LCD

#if LCD_CHIP == EPSON
    LCDCMD  VOLCTR  ; contrast setting is 0..63
    rrncf   LCD_contrast, w
    rrncf   WREG, w
    andlw   0x3F
    rcall   do_LCD_data
    LCDDATA 0x03    ; internal resistor ratio - coarse adjustment
#endif

#if LCD_CHIP == PHILIPS
    LCDCMD  VOLCTR  ; contrast setting is 0..127
    rrncf   LCD_contrast, w
    andlw   0x7F
    btfss   WREG,B6
    bra     LCD_set_contrast_go
    btfsc   WREG,B5
    andlw   b'01100000' ; limit to about 80% actual max contrast value
LCD_set_contrast_go
    rcall   do_LCD_data
#endif
    return


;----------------------------------------------------------------------
; blank the screen
;----------------------------------------------------------------------

LCD_cls ; optimized screen blank

    movlb   BANK_DISPLAY        ; access to banked variables for this subsystem

    ; prepare the LCD chip to receive a full screen of pixels [8 or 12 bpp]

    LCD_SPI         ; set up SPI port for LCD

    LCDCMD  PASET
    LCDDATA 0
    LCDDATA 131          
  
    LCDCMD  CASET
    LCDDATA 0        
    LCDDATA 131

    LCDCMD  RAMWR

#if LCD_CHIP == EPSON
    ; blank full screen (132x132 = 17424)
    ;   9 bits/pixel, but SPI shifts out 8 at a time
    ;   17424 *(9/8) = 19602 = 99 * 198
    ; using burst mode for interrupt mutex...
    ;   burst 8 pixels (9 bytes) at a time, then allow interrupts
    ;   9 8-bit writes = 72 bits = 8 pixels
    ;   19602/9 = 2178 blocks => 198 * 11
    MOVLFB  11, DISProws    ; 11 * 198 * 9 = 19602
blank_row_loop
    MOVLFB  198,DISPcols
blank_col_loop
    MOVLFB   9, DISPtemp
    bsf     PIR1, SSPIF
    bcf     LCD_CS
    movf    DISPbgnd, w, BANKED
    bsf     STATUS, C   ; data indicator
blank_block_loop
    rrcf    WREG, w     ; 9 bit rotate
blank_ssp_wait
    btfss   PIR1, SSPIF
    bra     blank_ssp_wait
    bcf     PIR1, SSPIF
    movwf   SSPBUF
    decfsz  DISPtemp, f, BANKED
    bra     blank_block_loop
    ; prepare to release SPI
blank_ssp_end_wait
    btfss   PIR1, SSPIF
    bra     blank_ssp_end_wait
    bsf     LCD_CS
    decfsz  DISPcols, f, BANKED
    bra     blank_col_loop
    decfsz  DISProws, f, BANKED
    bra     blank_row_loop
#endif

#if LCD_CHIP == PHILIPS
    ; blank full screen (132x132 = 17424 pixels = 26136 bytes)
    ;   9 bits/pixel, but SPI shifts out 8 at a time
    ;   26136 *(9/8) = 29403

    ; HEAVILY optimized for SLOW 12 bits/pixel interface

    ;  rrrrgggg  bbbbrrrr  ggggbbbb
    ; 0rrrrgggg 0bbbbrrrr 0ggggbbbb  0rrrrgggg 0bbbbrrrr 0ggggbbbb  0rrrrgggg 0bbbbrrrr 0ggggbbbb
    ; 123456781 234567812 345678123  456781234 567812345 678123456  781234567 812345678
    ; 111111112 222222233 333333444  444445555 555566666 666777777  77888888  899999999

    ; 11111111 22222222 33333333 444xxxxx
    ; 0rrrrggg g0bbbbrr rr0ggggb bbb
    ;
    ; bbb0rrrr gggg0bbb brrrr0gg ggb
    ;
    ; ggbbbb0r rrrgggg0 bbbbrrrr 0gg
    ;
    ; 0ggggbbb b0rrrrgg gg0bbbbr rrr
    ;
    ; rrr0gggg bbbb0rrr rgggg0bb bbr
    ;
    ; bbrrrr0g gggbbbb0 rrrrgggg 0bb
    ;
    ; 0bbbbrrr r0ggggbb bb0rrrrg ggg
    ;
    ; ggg0bbbb rrrr0ggg gbbbb0rr rrg
    ;
    ; rrgggg0b bbbrrrr0 ggggbbbb 0rr
    ;
    ; 26136 / 3 = 8172 => /9 = 968

    ; DISPbgnd == 12 bitz
    ; prepare 27 bit shift register
    ;   byte 1
    rrncf   DISPbgnd+1, w, BANKED   ; red+grn
    bsf     WREG, B7
    movwf   CLS_sr+0,      BANKED
    ;   byte 2
    rrncf   DISPbgnd+0, w, BANKED   ; blue
    rrncf   WREG, w
    andlw   b'00111100'
    bsf     WREG, B6
    movwf   CLS_sr+1,      BANKED
    rrncf   DISPbgnd+1, w, BANKED   ; grn
    andlw   b'10000000'
    iorwf   CLS_sr+1,      BANKED
    rlncf   DISPbgnd+1, w, BANKED   ; red
    rlncf   WREG, w
    andlw   b'00000011'
    iorwf   CLS_sr+1,      BANKED
    ; byte 3
    rlncf   DISPbgnd+1, w, BANKED   ; red
    rlncf   WREG, w
    andlw   b'11000000'
    bsf     WREG, B5
    movwf   CLS_sr+2,      BANKED
    rlncf   DISPbgnd+1, w, BANKED   ; green
    andlw   b'00011110'
    iorwf   CLS_sr+2,      BANKED
    rlncf   DISPbgnd+0, w, BANKED   ; blue
    andlw   b'00000001'
    iorwf   CLS_sr+2,      BANKED
    ; byte 4
    rlncf   DISPbgnd+0, w, BANKED   ; blue
    movwf   CLS_sr+3,      BANKED

    movff   CLS_sr+0, CLS_sra+0     ; 1
    movff   CLS_sr+1, CLS_sra+1
    movff   CLS_sr+2, CLS_sra+2
    rcall   shifty
    movff   CLS_sr+0, CLS_sra+3     ; 2
    movff   CLS_sr+1, CLS_sra+4
    movff   CLS_sr+2, CLS_sra+5
    rcall   shifty
    movff   CLS_sr+0, CLS_sra+6     ; 3
    movff   CLS_sr+1, CLS_sra+7
    movff   CLS_sr+2, CLS_sra+8
    rcall   shifty
    movff   CLS_sr+0, CLS_sra+9     ; 4
    movff   CLS_sr+1, CLS_sra+10
    movff   CLS_sr+2, CLS_sra+11
    rcall   shifty
    movff   CLS_sr+0, CLS_sra+12    ; 5
    movff   CLS_sr+1, CLS_sra+13
    movff   CLS_sr+2, CLS_sra+14
    rcall   shifty
    movff   CLS_sr+0, CLS_sra+15    ; 6
    movff   CLS_sr+1, CLS_sra+16
    movff   CLS_sr+2, CLS_sra+17
    rcall   shifty
    movff   CLS_sr+0, CLS_sra+18    ; 7
    movff   CLS_sr+1, CLS_sra+19
    movff   CLS_sr+2, CLS_sra+20
    rcall   shifty
    movff   CLS_sr+0, CLS_sra+21    ; 8
    movff   CLS_sr+1, CLS_sra+22
    movff   CLS_sr+2, CLS_sra+23
    rcall   shifty
    movff   CLS_sr+0, CLS_sra+24    ; 9
    movff   CLS_sr+1, CLS_sra+25
    movff   CLS_sr+2, CLS_sra+26

CLS_EMIT macro zData
    local ssp_wait
    movf    zData, w, BANKED
ssp_wait
    btfss   PIR1, SSPIF
    bra     ssp_wait
    bcf     PIR1, SSPIF
    movwf   SSPBUF
    endm

    MOVLFB    9,DISProws            ; 29403 = 27 * 9 * 121
blank_emit_megablok_loop
    MOVLFB  121,DISPcols
blank_emit_blok9_loop
    bcf     LCD_CS
    bsf     PIR1, SSPIF
    CLS_EMIT CLS_sra+0
    CLS_EMIT CLS_sra+1
    CLS_EMIT CLS_sra+2
    CLS_EMIT CLS_sra+3
    CLS_EMIT CLS_sra+4
    CLS_EMIT CLS_sra+5
    CLS_EMIT CLS_sra+6
    CLS_EMIT CLS_sra+7
    CLS_EMIT CLS_sra+8
    CLS_EMIT CLS_sra+9
    CLS_EMIT CLS_sra+10
    CLS_EMIT CLS_sra+11
    CLS_EMIT CLS_sra+12
    CLS_EMIT CLS_sra+13
    CLS_EMIT CLS_sra+14
    CLS_EMIT CLS_sra+15
    CLS_EMIT CLS_sra+16
    CLS_EMIT CLS_sra+17
    CLS_EMIT CLS_sra+18
    CLS_EMIT CLS_sra+19
    CLS_EMIT CLS_sra+20
    CLS_EMIT CLS_sra+21
    CLS_EMIT CLS_sra+22
    CLS_EMIT CLS_sra+23
    CLS_EMIT CLS_sra+24
    CLS_EMIT CLS_sra+25
    CLS_EMIT CLS_sra+26
blank_ssp_end_wait
    btfss   PIR1, SSPIF
    bra     blank_ssp_end_wait
    bsf     LCD_CS
    decfsz  DISPcols, f, BANKED
    bra     blank_emit_blok9_loop
    decfsz  DISProws, f, BANKED
    bra     blank_emit_megablok_loop
#endif

    LCDCMD  LCD_NOP

;-- turn on backlight AFTER first CLS is complete

    LCD_SET_BACKLIGHT
    return


#if LCD_CHIP == PHILIPS
shifty
    ; 27 bit shifter
    swapf   CLS_sr+3, f, BANKED     ; 'bbb_____' => '____bbb_'
    rrcf    CLS_sr+3, f, BANKED     ; '____bbb_' => '_____bbb'
    rrcf    CLS_sr+3, f, BANKED     ; preload CY
    ; shift 1
    rrcf    CLS_sr+0, f, BANKED
    rrcf    CLS_sr+1, f, BANKED
    rrcf    CLS_sr+2, f, BANKED
    rrcf    CLS_sr+3, f, BANKED
    ; shift 2
    rrcf    CLS_sr+0, f, BANKED
    rrcf    CLS_sr+1, f, BANKED
    rrcf    CLS_sr+2, f, BANKED
    rrcf    CLS_sr+3, f, BANKED
    ; shift 3
    rrcf    CLS_sr+0, f, BANKED
    rrcf    CLS_sr+1, f, BANKED
    rrcf    CLS_sr+2, f, BANKED
    rrcf    CLS_sr+3, f, BANKED
    return
#endif


;----------------------------------------------------------------------
; display 1 pixel
;----------------------------------------------------------------------

LCD_putpixel    ; caller MUST preload bank register!

    ; set X
    movf    DISPxaset,   w, BANKED
    rcall   do_LCD_command
    movf    DISPx, w,       BANKED
    addwf   DISPxOffset, w, BANKED
    movwf   DISPtemp,       BANKED
    rcall   do_LCD_data
    movf    DISPtemp, w,    BANKED
    rcall   do_LCD_data
    ; set Y
    movf    DISPyaset,   w, BANKED
    rcall   do_LCD_command
    movf    DISPy, w,       BANKED
    addwf   DISPyOffset, w, BANKED
    movwf   DISPtemp,       BANKED
    rcall   do_LCD_data
    movf    DISPtemp, w,    BANKED
    rcall   do_LCD_data
    ; paint the pixel
    LCDCMD  RAMWR       ; can now send pixels, LCD will autoincrement address

#if LCD_CHIP == EPSON
    ;   8 bits/pixel
    movf    DISPfgnd, w,    BANKED
    bra     do_LCD_data
#endif

#if LCD_CHIP == PHILIPS
    ; 12 bits/pixel, 2 pixels at a time: rrrrgggg bbbbrrrr ggggbbbb
    ;   1'st byte                        -byte 1- -byte 2- -byte 3-
    movf    DISPfgnd+1, w,  BANKED
    rcall   do_LCD_data
    ;   2'nd byte
    swapf   DISPfgnd+1, w,  BANKED   ; get red
    andlw   b'00001111'
    iorwf   DISPfgnd+0, w,  BANKED   ; + blue
    rcall   do_LCD_data
    ; ABORT the 2'nd pixel
    LCDCMD  LCD_NOP     ; end write
    return
#endif


;----------------------------------------------------------------------
; display a "page" on the LCD
;----------------------------------------------------------------------

display_cmd_err     ; handle screwed up page driver data (or trashed page display pointer)

    SET_ERROR_FREEZE E_DISPLAY
    btfsc   pgm_flags, _FLAG_ERROR
    reset   ; nested error condition
    bsf     pgm_flags, _FLAG_ERROR
            ; single error condition
disp_end
    return  

; - - - - - - - - - - - - - - - - - - - - -

LCD_display_page    ; from data at TBLPTR

    movlb   BANK_DISPLAY    ; access to banked variables for this subsystem

    LCD_SPI         ; set up SPI port for LCD

    ; default to transparent mode
    clrf    DISPmode, BANKED

    ; clear error nest
    bcf     pgm_flags, _FLAG_ERROR

;-- display data interpreter

display_cmd_loop

    tblrd*+                 ; get command byte
    ; convergent programming, inc.
    movlw   20+1
    cpfslt  TABLAT    
    bra     display_cmd_err
    ; jump table
    MOVLF   high(disp_cmd_table), PCLATH
    rlncf   TABLAT, w       ; 2 bytes/instruction
    tblrd*+                 ; prefetch data byte
    addlw   low(disp_cmd_table)
    btfsc   STATUS, C
    incf    PCLATH, f
    movwf   PCL
disp_cmd_table
    bra     disp_end        ; 0
    bra     disp_at         ; 1
    bra     disp_color      ; 2
    bra     disp_bgnd       ; 3
    bra     disp_vcolor     ; 4
    bra     disp_font       ; 5
    bra     disp_Hline      ; 6
    bra     disp_Hdash      ; 7
    bra     disp_Vline      ; 8
    bra     disp_var        ; 9
    bra     disp_char       ; 10
    bra     disp_string     ; 11
    bra     disp_erase      ; 12
    bra     disp_if         ; 13
    bra     disp_mode       ; 14
    bra     disp_pixel      ; 15
    bra     disp_ifbit      ; 16
    bra     disp_branch     ; 17
    bra     disp_cls        ; 18
    bra     disp_aray       ; 19
    bra     disp_fcn        ; 20

;-- display command handlers

disp_cls    ; force screen clear [on next Backgnd command encountered]
    bsf     pgm_flags, _FLAG_CLS
    bra     display_cmd_loop

disp_at
    ; constrain X (LCD rows) from current position to end of screen
    tblrd*+ ; get X value
    movff   TABLAT, DISPx
    movf    DISPxaset,w,    BANKED
    rcall   do_LCD_command 
    movf    DISPx, w,       BANKED
    addwf   DISPxOffset, w, BANKED
    rcall   do_LCD_data
    LCDDATA 131
    ; constrain Y (LCD columns) to character height
    tblrd*+ ; get Y value
    movff   TABLAT, DISPy
    movf    DISPyaset, w,    BANKED
    rcall   do_LCD_command
    decf    FONT_Hpixels, w, BANKED
    subwf   DISPy, w,        BANKED
    addwf   DISPyOffset, w,  BANKED
    call    do_LCD_data
    movf    DISPy, w,        BANKED
    addwf   DISPyOffset, w,  BANKED
    call    do_LCD_data
    ; now ready to emit pixel columns
    LCDCMD  RAMWR       ; can now send full rows, LCD will autoincrement address
    bra     display_cmd_loop

disp_font ; look up font parameters in font table

    movff   TBLPTRL, DISPsave+0
    movff   TBLPTRH, DISPsave+1

    MOVLF   high(FONT_TABLE), TBLPTRH
    MOVLF    low(FONT_TABLE), TBLPTRL
    movf    TABLAT, w           ; font table offset
    addwf   TBLPTRL, f
    movlw   0
    addwfc  TBLPTRH, f

    tblrd*+
    movff   TABLAT, FONT_Hpixels
    tblrd*+
    tblrd*+
    movff   TABLAT, FONT_Bitmaps+0
    tblrd*+
    movff   TABLAT, FONT_Bitmaps+1
    tblrd*+
    movff   TABLAT, FONT_Sizes+0
    tblrd*+
    movff   TABLAT, FONT_Sizes+1

    movff   DISPsave+0, TBLPTRL
    movff   DISPsave+1, TBLPTRH

    bra     display_cmd_loop

disp_mode
    movff   TABLAT, DISPmode
    bra     display_cmd_loop

disp_color
    MOVLF   BANK_DISPLAY, FSR0H ; set color palette bank
    movff   TABLAT,       FSR0L ;   INDF0 now pointing to actual color
    movff   POSTINC0, DISPfgnd+0
    movff   POSTINC0, DISPfgnd+1
    bra     display_cmd_loop

disp_bgnd
    MOVLF   BANK_DISPLAY, FSR0H ; set color palette bank
    movff   TABLAT,       FSR0L ;   INDF0 now pointing to actual color
    movff   POSTINC0, DISPbgnd+0
    movff   POSTINC0, DISPbgnd+1
    ; conditionally clear the screen to the backgnd color
    btfsc   pgm_flags, _FLAG_CLS
    rcall   LCD_cls
    bcf     pgm_flags, _FLAG_CLS
    bra     display_cmd_loop

disp_vcolor
    MOVLF   BANK_DISPLAY, FSR0H ; set display variable bank
    movff   TABLAT,       FSR0L ;   INDF0 now pointing to variable data
    movff   INDF0, FSR0L        ; retrieve value [color palette location]
    MOVLF   BANK_DISPLAY, FSR0H ; set color palette bank
    movff   POSTINC0, DISPfgnd+0
    movff   POSTINC0, DISPfgnd+1
    bra     display_cmd_loop

disp_char
    movf    TABLAT, w
    rcall   disp_putchar
    bra     display_cmd_loop

disp_string
    movff   TABLAT, DISPchars
disp_string_loop
    tblrd*+ ; read char stream
    movf    TABLAT, w
    rcall   disp_putchar
    decfsz  DISPchars, f, BANKED
    bra     disp_string_loop
    ; realign on word boundary
    btfsc   TBLPTRL, B0
    tblrd*+
    bra     display_cmd_loop

disp_aray
    tblrd*+ ; skip filler byte
    movff   TABLAT, FSR0L       ; point to variable [must be a byte]
    tblrd*+
    movff   TABLAT, FSR0H
    ; override table pointer to get at string table
    tblrd*+ ; get table address (lo byte)
    movff   TABLAT, DISPtemp
    tblrd*+ ; get table address (hi byte)
    movff   TBLPTRL, DISPstak+0 ; pointing to next page script command
    movff   TBLPTRH, DISPstak+1
    movff   TABLAT,   TBLPTRH
    movff   DISPtemp, TBLPTRL   ; now pointing to string table start
    ;   scan string table
    tblrd*+ ; get number_of_elements
    movff   TABLAT, DISProws    ; this value is ignored !!!
    tblrd*+ ; get print_width
    movff   TABLAT, DISPchars   ; now pointing to 1'st string element
    ;   calc offset into table
    movf    DISPchars, w, BANKED
    btfsc   WREG, B0
    addlw   1                   ; odd string length rounds up to even word boundary
    mulwf   INDF0
    movf    PRODL, w            ; WARNING - offsets > 255 are NOT SUPPORTED
    CALC_TABLE_ADDRESS          ; TBLPTR += WREG, pointing to start of string
    ;   emit string
disp_aray_loop
    tblrd*+ ; read char stream
    movf    TABLAT, w
    rcall   disp_putchar        ; destroys DISP* except DISPchars & DISPstak, and preserves TBLPTR
    decfsz  DISPchars, f, BANKED
    bra     disp_aray_loop
    ;   restore table pointer
    movff   DISPstak+0, TBLPTRL
    movff   DISPstak+1, TBLPTRH
    bra     display_cmd_loop

disp_var
    MOVLF   BANK_DISPLAY, FSR0H ; set display variable bank
    movff   TABLAT,       FSR0L ;   INDF0 now pointing to variable data
    movf    INDF0, w            ; retrieve value
    rcall   disp_putchar
    bra     display_cmd_loop

disp_if ; [ifne]
    movff   TABLAT, DISPtemp    ; save constant value
    tblrd*+ ; get variable location (lo byte)
    movff   TABLAT,       FSR0L
    tblrd*+ ; get variable location (hi byte)
    movff   TABLAT,       FSR0H ;   INDF0 now pointing to variable data
    movf    INDF0, w            ; retrieve value
    cpfseq  DISPtemp            ;   compare to constant
    bra     disp_branch
    ; discard target address
disp_if_not
    tblrd*+
    tblrd*+
    bra     display_cmd_loop
disp_branch
    ; reset text pointer to target address
    tblrd*+
    movf    TABLAT, w           ; get low byte of target
    tblrd*+
    movff   TABLAT, TBLPTRH     ; get high byte of target
    movwf           TBLPTRL
    bra     display_cmd_loop

disp_ifbit
    tblrd*+ ; get variable location (lo byte)
    movff   TABLAT,       FSR0L
    tblrd*+ ; get variable location (hi byte)
    movff   TABLAT,       FSR0H ;   INDF0 now pointing to variable data
    movf    INDF0, w            ; retrieve value
    tblrd*+ ; get bit_mask
    andwf   TABLAT, w
    tblrd*+ ; get bit_xor
    xorwf   TABLAT, w
    bz      disp_branch
    bra     disp_if_not

disp_pixel
    movff   TABLAT, DISPx
    tblrd*+
    movff   TABLAT, DISPy
    tblrd*+
    rcall   LCD_putpixel
    bra     display_cmd_loop

disp_fcn
    rcall   disp_fcn_subr       ; user function must "return"
    bra     display_cmd_loop
disp_fcn_subr
    tblrd*+
    movf    TABLAT, w           ; get low byte of target
    tblrd*+
    movff   TABLAT, PCLATH      ; get high byte of target
    movwf           PCL

;----------------------------------------------------------------------

; line lengths must always be a multiple of 2 (for Philips chip)

disp_Hline
    ; set Y
    movf    DISPyaset,   w, BANKED
    rcall   do_LCD_command
    movf    TABLAT, w
    addwf   DISPyOffset, w, BANKED
    rcall   do_LCD_data
    movf    TABLAT, w
    addwf   DISPyOffset, w, BANKED
    rcall   do_LCD_data
    ; set X
    movf    DISPxaset,   w, BANKED
    rcall   do_LCD_command
    tblrd*+ ; read x0
    movf    TABLAT, w
    addwf   DISPxOffset, w, BANKED
    movwf   DISPtemp,       BANKED
    rcall   do_LCD_data
    tblrd*+ ; read line length
    movf    TABLAT, w
    movwf   DISPcols,       BANKED
    addwf   DISPtemp, w,    BANKED
    decf    WREG, w
    rcall   do_LCD_data

disp_line_common
    LCDCMD  RAMWR       ; can now send pixels, LCD will autoincrement address
#if LCD_CHIP == EPSON
disp_line_loop
    movf    DISPfgnd, w, BANKED
    rcall   do_LCD_data
#endif
#if LCD_CHIP == PHILIPS
    rrcf    DISPcols, f,  BANKED
    bcf     DISPcols, B7, BANKED
disp_line_loop
    ; 12 bits/pixel, 2 pixels at a time: rrrrgggg bbbbrrrr ggggbbbb
    ;   1'st byte                        -byte 1- -byte 2- -byte 3-
    movf    DISPfgnd+1, w, BANKED
    rcall   do_LCD_data
    ;   2'nd byte
    swapf   DISPfgnd+1, w, BANKED   ; get red
    andlw   b'00001111'
    iorwf   DISPfgnd+0, w, BANKED   ; + blue
    rcall   do_LCD_data
    ;   3'rd byte
    movf    DISPfgnd+1, w, BANKED   ; grn
    andlw   b'00001111'
    iorwf   DISPfgnd+0, w, BANKED   ; blue
    swapf   WREG, w
    rcall   do_LCD_data
#endif
    decfsz  DISPcols, f,   BANKED
    bra     disp_line_loop
    bra     display_cmd_loop


disp_Vline
    ; set X
    movf    DISPxaset,   w, BANKED
    rcall   do_LCD_command
    movf    TABLAT, w               ; x
    addwf   DISPxOffset, w, BANKED
    rcall   do_LCD_data
    movf    TABLAT, w
    addwf   DISPxOffset, w, BANKED
    rcall   do_LCD_data
    ; set Y
    movf    DISPyaset,   w, BANKED
    rcall   do_LCD_command
    tblrd*+ ; read y0
    movf    TABLAT, w               ; y
    addwf   DISPyOffset, w, BANKED
    movwf   DISPtemp,       BANKED
    rcall   do_LCD_data
    tblrd*+ ; read line length
    movf    TABLAT, w               ; length
    movwf   DISPcols,       BANKED
    addwf   DISPtemp, w,    BANKED
    decf    WREG, w
    rcall   do_LCD_data
    bra     disp_line_common


disp_Hdash
    ; set Y
    movf    DISPyaset,   w, BANKED
    rcall   do_LCD_command
    movf    TABLAT, w
    addwf   DISPyOffset, w, BANKED
    rcall   do_LCD_data
    movf    TABLAT, w
    addwf   DISPyOffset, w, BANKED
    rcall   do_LCD_data
    ; set X
    movf    DISPxaset,   w, BANKED
    rcall   do_LCD_command
    tblrd*+ ; read x0
    movf    TABLAT, w
    addwf   DISPxOffset, w, BANKED
    movwf   DISPtemp,       BANKED
    rcall   do_LCD_data
    tblrd*+ ; read line length
    movf    TABLAT, w
    movwf   DISPcols,       BANKED
    addwf   DISPtemp, w,    BANKED
    decf    WREG, w
    rcall   do_LCD_data
    ; get dash style
    tblrd*+ ; point to style
    ; draw the line
    LCDCMD  RAMWR       ; can now send pixels, LCD will autoincrement address
#if LCD_CHIP == EPSON
disp_Hdash_loop
    movff   TABLAT, DISProws
disp_Hdash_draw_loop
    movf    DISPfgnd, w,   BANKED
    rcall   do_LCD_data
    dcfsnz  DISPcols, f,   BANKED
    bra     disp_Hdash_done
    decfsz  DISProws, f,   BANKED
    bra     disp_Hdash_draw_loop
    ; dashed part
    movff   TABLAT, DISProws
disp_Hdash_nodraw_loop
    movf    DISPbgnd, w,   BANKED
    rcall   do_LCD_data
    dcfsnz  DISPcols, f,   BANKED
    bra     disp_Hdash_done
    decfsz  DISProws, f,   BANKED
    bra     disp_Hdash_nodraw_loop
    bra     disp_Hdash_loop
disp_Hdash_done
    tblrd*+ ; align on word boundary
#endif
#if LCD_CHIP == PHILIPS
;error NOT IMPLEMENTED
#endif
    bra     display_cmd_loop

;----------------------------------------------------------------------

; number of rows must always be a multiple of 2 (for Philips chip)

disp_erase   ; block clear to backgnd color

    ; constrain Y (the LCD thinks of these as columns)
    movf    DISPyaset,   w, BANKED
    rcall   do_LCD_command
    tblrd*+ ; read y
    movf    TABLAT, w
    movwf   DISPtemp, BANKED
    addwf   DISPyOffset, w, BANKED
    rcall   do_LCD_data
    tblrd*+ ; read height
    movf    TABLAT, w
    movwf   DISProws, BANKED
    addwf   DISPtemp, w, BANKED
    decf    WREG, w
    addwf   DISPyOffset, w, BANKED
    rcall   do_LCD_data
    ; set starting X (LCD row)
    movf    DISPxaset,   w, BANKED
    rcall   do_LCD_command
    tblrd*+ ; read x
    movf    TABLAT, w
    addwf   DISPxOffset, w, BANKED
    rcall   do_LCD_data
    LCDDATA 131 ; unnecessary to constrain ending X coordinate
    tblrd*+ ; read width
    movf    TABLAT, w
    movwf   DISPcols, BANKED

    LCDCMD  RAMWR       ; can now send pixels, LCD will autoincrement address
disp_erase_col_loop
#if LCD_CHIP == EPSON
    movff   DISProws, DISPtemp
disp_erase_row_loop
    movf    DISPbgnd, w, BANKED
    rcall   do_LCD_data
#endif
#if LCD_CHIP == PHILIPS
    rrcf    DISProws, w, BANKED
    bcf     WREG, B7
    movwf   DISPtemp, BANKED
disp_erase_row_loop
    ; 12 bits/pixel, 2 pixels at a time: rrrrgggg bbbbrrrr ggggbbbb
    ;   1'st byte                        -byte 1- -byte 2- -byte 3-
    movf    DISPbgnd+1, w, BANKED   ; red+grn
    rcall   do_LCD_data
    ;   2'nd byte
    swapf   DISPbgnd+1, w, BANKED   ; 2'nd pixel red
    andlw   b'00001111'
    iorwf   DISPbgnd+0, w, BANKED   ; + blue
    rcall   do_LCD_data
    ;   3'rd byte
    movf    DISPbgnd+1, w, BANKED   ; grn
    andlw   b'00001111'
    iorwf   DISPbgnd+0, w, BANKED   ; blue
    swapf   WREG, w
    rcall   do_LCD_data
#endif
    decfsz  DISPtemp, f, BANKED
    bra     disp_erase_row_loop
    decfsz  DISPcols, f, BANKED
    bra     disp_erase_col_loop
    bra     display_cmd_loop

;----------------------------------------------------------------------

; before calling this function, AT must executed to start the RAMWR

disp_putchar ; from WREG

    addlw   -0x21               ; make char zero offset
    movwf   DISPtemp, BANKED    ; save char
    bnc     disp_putchar_spacer ; values less that 0x21 are spacers

    ; preserve table pointer
    movff   TBLPTRL, DISPsave+0
    movff   TBLPTRH, DISPsave+1

    ; get the char width
    movff   FONT_Sizes+0, TBLPTRL
    movff   FONT_Sizes+1, TBLPTRH
    movf    DISPtemp, w, BANKED
    CALC_TABLE_ADDRESS
    tblrd*
    movf    TABLAT, w
    movwf   DISPcols, BANKED
    ; bail out if char doesn't exist (zero width)
    bz      disp_putchar_exit

    ; point to start of char data
    movff   FONT_Bitmaps+0, TBLPTRL
    movff   FONT_Bitmaps+1, TBLPTRH
    rlncf   DISPtemp, w, BANKED ; char * 2
    CALC_TABLE_ADDRESS
    ; indirect to actual data location
    tblrd*+
    movf    TABLAT, w   ; low byte of char address
    tblrd*
    movwf   TBLPTRL
    movf    TABLAT, w   ; high byte of char address
    movwf   TBLPTRH
    ; add bitmap table base address to offset
    movff   FONT_Bitmaps+0, WREG
    addwf   TBLPTRL, f
    movff   FONT_Bitmaps+1, WREG
    addwfc  TBLPTRH, f

    ; increment display position [the display loop will destroy DISPcols]
    movf    DISPcols, w, BANKED
    addwf   DISPx, f, BANKED

    ; only emit actual char height (may be less than a byte multiple)
disp_putchar_col_loop
    movff   FONT_Hpixels, DISPpixels
disp_putchar_byte_loop
    tblrd*+
    movff   TABLAT, DISPscan

#if LCD_CHIP == EPSON
    MOVLFB  8, DISProws
disp_putchar_bit_loop
    movf    DISPbgnd, w, BANKED
    rlcf    DISPscan, f, BANKED
    btfsc   STATUS, C
    movf    DISPfgnd, w, BANKED
    rcall   do_LCD_data
    dcfsnz  DISPpixels, f, BANKED
    bra     disp_putchar_endcol
    decfsz  DISProws, f, BANKED
    bra     disp_putchar_bit_loop
    bra     disp_putchar_byte_loop
#endif

#if LCD_CHIP == PHILIPS
    ; 12 bits/pixel, 2 pixels at a time: rrrrgggg bbbbrrrr ggggbbbb
    ;   1'st byte                        -byte 1- -byte 2- -byte 3-
    MOVLFB  4, DISProws
disp_putchar_dibit_loop
    ; 1'st pixel
    rlcf    DISPscan, f, BANKED
    bc      disp_putchar_dibit_1_fgnd
    movff   DISPbgnd+0, DISPtemp    ; blue
    movf    DISPbgnd+1, w, BANKED   ; red+grn
    bra     disp_putchar_dibit_1
disp_putchar_dibit_1_fgnd
    movff   DISPfgnd+0, DISPtemp    ; blue
    movf    DISPfgnd+1, w, BANKED   ; red+grn
disp_putchar_dibit_1
    rcall   do_LCD_data
    ; 2'nd pixel
    rlcf    DISPscan, f, BANKED
    bc      disp_putchar_dibit_2_fgnd
    swapf   DISPbgnd+1, w, BANKED   ; red
    andlw   b'00001111'
    iorwf   DISPtemp, w, BANKED     ; + blue from 1'st pixel
    rcall   do_LCD_data
    movf    DISPbgnd+1, w, BANKED   ; grn
    andlw   b'00001111'
    iorwf   DISPbgnd+0, w, BANKED   ; blue
    bra     disp_putchar_dibit_byte3
disp_putchar_dibit_2_fgnd
    swapf   DISPfgnd+1, w, BANKED   ; red
    andlw   b'00001111'
    iorwf   DISPtemp, w, BANKED     ; + blue from 1'st pixel
    rcall   do_LCD_data
    movf    DISPfgnd+1, w, BANKED   ; grn
    andlw   b'00001111'
    iorwf   DISPfgnd+0, w, BANKED   ; blue
disp_putchar_dibit_byte3
    swapf   WREG, w
    rcall   do_LCD_data
    ;loop
    decf    DISPpixels, f, BANKED
    dcfsnz  DISPpixels, f, BANKED
    bra     disp_putchar_endcol
    decfsz  DISProws, f, BANKED
    bra     disp_putchar_dibit_loop
    bra     disp_putchar_byte_loop
#endif

disp_putchar_endcol
    decfsz  DISPcols, f, BANKED
    bra     disp_putchar_col_loop

    ; advance cursor to next text output position
    movlw   2-0x21      ; character spacing
    rcall   disp_putchar_spacer

disp_putchar_exit

    ; restore table pointer
    movff   DISPsave+0, TBLPTRL
    movff   DISPsave+1, TBLPTRH

    return


disp_putchar_spacer
    ; handle spacer control code (char value = number of pixels to skip)
    addlw   0x21            ; restore char code
    addwf   DISPx, f, BANKED
    ; transparent vs. overstrike mode
    btfsc   DISPmode, B0, BANKED
    bra     disp_putchar_spacer_overstrike
    ; transparent spacer, just reposition the output cursor to skip the blank rows
    movf    DISPxaset,   w, BANKED
    rcall   do_LCD_command
    movf    DISPx, w, BANKED
    addwf   DISPxOffset, w, BANKED
    rcall   do_LCD_data
    LCDDATA 131
    LCDCMD  RAMWR       ; can now send full rows, LCD will autoincrement address
    return

disp_putchar_spacer_overstrike
    ; overstrike spacer, write blank rows with backgnd color
    movwf   DISPcols, BANKED
    ; emit blank columns
disp_putchar_spacer_col_loop
#if LCD_CHIP == EPSON
    movff   FONT_Hpixels, DISPpixels
disp_putchar_spacer_row_loop
    movf    DISPbgnd, w, BANKED
    rcall   do_LCD_data
#endif
#if LCD_CHIP == PHILIPS
    rrncf   FONT_Hpixels, w, BANKED
    bcf     WREG, B7
    movwf   DISPpixels, BANKED
disp_putchar_spacer_row_loop
    ; 12 bits/pixel, 2 pixels at a time: rrrrgggg bbbbrrrr ggggbbbb
    ;   1'st byte                        -byte 1- -byte 2- -byte 3-
    movf    DISPbgnd+1, w, BANKED
    rcall   do_LCD_data
    ;   2'nd byte
    swapf   DISPbgnd+1, w, BANKED   ; get red
    andlw   b'00001111'
    iorwf   DISPbgnd+0, w, BANKED   ; + blue
    rcall   do_LCD_data
    ;   3'rd byte
    movf    DISPbgnd+1, w, BANKED   ; grn
    andlw   b'00001111'
    iorwf   DISPbgnd+0, w, BANKED   ; blue
    swapf   WREG, w
    rcall   do_LCD_data
#endif
    decfsz  DISPpixels, f, BANKED
    bra     disp_putchar_spacer_row_loop
    decfsz  DISPcols, f, BANKED
    bra     disp_putchar_spacer_col_loop
    return


;----------------------------------------------------------------------
; LCD color palettes
;----------------------------------------------------------------------

#if LCD_CHIP == PHILIPS
            ; rrrrggggbbbb----
BLK     EQU b'0000000000000000' ;good
WHI     EQU b'1111111111110000' ;good
GRY     EQU b'0111011101110000' ;good
;             rrrrggggbbbb
RED     EQU b'1111000000000000' ;good
GRN     EQU b'0000011100000000' ;good
BLU     EQU b'0000000011110000' ;good
YEL     EQU b'1110100001000000'
;             rrrrggggbbbb
DK_GRN  EQU b'0000011100000000' ;good
DK_YEL  EQU b'0111011101110000' ;CRAPPY
DK_RED  EQU b'0011000000000000'
DK_BLU  EQU b'0000000000110000' ;good
;             rrrrggggbbbb
LT_GRY  EQU b'0001000100010000'
LT_GRN  EQU b'0001111100010000' ;good
LT_BLU  EQU b'0001000111110000'
LT_RED  EQU b'1111000100010000' ;bright red
;             rrrrggggbbbb
BRT_GRN EQU b'0000111100000000'
DKR_GRN EQU b'0001011100010000'
;             rrrrggggbbbb
PINK    EQU b'1111011101110000' ; ok
GOLD    EQU b'0111011100000000' ; ok
BRN     EQU b'0011001100110000' ; ok
BRN2    EQU b'1100100000010000'
MAG     EQU b'0101000000110000' ; ok [actually, crappy]
#endif

#if LCD_CHIP == EPSON
; monochrome colors
BLK     EQU 0
GRY     EQU b'10110110'
LT_GRY  EQU b'00100101'
WHI     EQU 255

MD_GRY  EQU 5
DK_GRY  EQU 9

; blues
BLU     EQU 2
DK_BLU  EQU 1

LT_BLU  EQU 3
LT1_BLU EQU 7
LT2_BLU EQU 10
LT3_BLU EQU 11
;MAG     EQU 131 [crappy]
MAG2    EQU 194
LT_MAG  EQU 207

; reds
RED     EQU b'11100000'
DK_RED  EQU b'01000000'
PINK    EQU b'11110010'

DK0_RED EQU 32
DK1_RED EQU 96
RED1    EQU 128
RED2    EQU 132
DK3_RED EQU 136
RED3    EQU 161
RED4    EQU 164
BRT_RED EQU 168
RED6    EQU 196
LT_RED  EQU 205
LT2_RED EQU 233

; greens      rrrgggbb
GRN     EQU b'00011100'
GRNx    EQU b'00010100'
LT_GRN  EQU b'00111101'
DK_GRN  EQU b'00001000'

; yellows
YEL     EQU 180
GOLD    EQU 176
DK_YEL  EQU b'10001110'
BRN     EQU b'01001001'

YEL2    EQU 185
LT_YEL  EQU 188
LT2_YEL EQU 220
LT3_YEL EQU 248
LT4_YEL EQU 252
#endif

palette_table

    ;   splash  splash  popup   main    main    main    high    copy    settings graph    graph
    ;   bgnd    text    text    bgnd    text    titles  light   right   text     voltage  current
    ;   ------- ------- ------- ------- ------- ------- ------- ------- -------- -------- --------
    dw  BLK,    WHI,    WHI,    BLK,    WHI,    LT_GRN, RED,    BLU,    WHI,     RED,     LT_GRN

LOAD_COLOR macro zPal
    tblrd*+
    movff   TABLAT, zPal+0
    tblrd*+
    movff   TABLAT, zPal+1
    endm

LCD_palette_load    ; load color palette [only 1 is available in this application]

    MOVLF   high(palette_table), TBLPTRH
    MOVLF    low(palette_table), TBLPTRL

    ; brute force data copy

    LOAD_COLOR PAL_splash_bgnd
    LOAD_COLOR PAL_splash_text
    LOAD_COLOR PAL_popup_text

    LOAD_COLOR PAL_main_bgnd
    LOAD_COLOR PAL_main_text
    LOAD_COLOR PAL_title_text
    LOAD_COLOR PAL_highlight

    LOAD_COLOR PAL_copyright

    LOAD_COLOR PAL_settings_text

    LOAD_COLOR PAL_graph_V
    LOAD_COLOR PAL_graph_I

    return
