;**********************************************************************
;
;    I N T E R R U P T S
;
;    (see primary interrupt file for requirements)
;
;**********************************************************************

;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Battery Monitor (analyze/charge/discharge logic)
;
; Entry conditions:
;   all int_temp variables are available for use by this module
;   FSR2 is available for use by this module
;----------------------------------------------------------------------
;----------------------------------------------------------------------

; temporary variables used by this module
#define temp int_temp ; requires [6] bytes

;----------------------------------------------------------------------
; Battery monitor - drives analyze/charge/discharge modes
;
; CALLED ONLY FROM HIGH PRIORITY INTERRUPT

batmon  ; poll at 2.5 msec rate

;-- get raw A/D B+ value from interrupt buffer

    MOV16   adc_data_aray+ADC_DATAINDEX_B1*2, temp+4  ; temp[4,5] = raw A/D value

;-- feed B+ values through filtering PID

    rcall   bat_input_PID       ; temp[4,5] => bplus_filt

;-- run active process

    btfss   bat_flags, _BAT_DIAGNOSTIC
    bra     batmon_normal

;-- active process is diagnostics

    ; feed filtered B+ to charge/load PID's (CV mode only)
    btfsc   bat_flags, _BAT_CHARGE_ACTIVE
    rcall   batmon_charger      ; bplus_filt(CV) => iChg_filt  => POT
    btfsc   bat_flags, _BAT_LOAD_ACTIVE
    bra     batmon_discharger   ; bplus_filt(CV) => iLoad_filt => POT

;-- run normal charger/analyzer logic

batmon_normal

    ; is battery loaded ?
    btfsc   bat_flags, _BAT_LOADED
    bra     batmon_loaded

;-- no battery loaded

    ; no battery - monitor for battery insert
    MOVLF16 MIN_BAT_VOLTAGE*2, temp ; ADC units
    CP16    bplus_filt,        temp
    bc      batmon_end    
    ; battery detected
    tstfsz  bat_debounce        ; == 0 if timer has not been started
    bra     batmon_load_debounce
    ;   start battery insert timer
    MOVLF   BAT_DEBOUNCE_TICKS, bat_debounce
batmon_end
    return

batmon_load_debounce
    decfsz  bat_debounce, f
    return
    ;   debounce complete
    bsf     bat_flags, _BAT_LOADED
    return

batmon_loaded

    btfsc   bat_flags, _BAT_CHARGE_ACTIVE
    bra     batmon_charging
    btfsc   bat_flags, _BAT_LOAD_ACTIVE
    bra     batmon_discharging 
    ; monitor for battery removed (while idle)
    ;   B+ will fall to ~0V when battery is removed
    MOVLF16 MIN_BAT_VOLTAGE*2, temp ; ADC units
    CP16    bplus_filt,        temp
    bc      batmon_unload
    return

batmon_unload
    bcf     bat_flags, _BAT_LOADED
    clrf    bat_debounce    ; reset battery insert debouncer
    LED_OFF LED_GRN
    LED_OFF LED_RED
    ; power down charger (immediately, no debounce)
    bra     batmon_operation_terminate ; & return

;-- charging logic

batmon_charging

    ; monitor for battery removed
    ;   B+ will rise to >5V when battery is removed (if charging)
    MOVLF16 NO_BAT_VOLTAGE, temp
    CP16    bplus_filt,     temp
    bnc     batmon_unload
    ; run charger logic
    bra     batmon_charger  ; & return

;-- discharging logic

batmon_discharging

    ; monitor for battery removed
    ;   B+ will fall to ~0V when battery is removed
    MOVLF16 MIN_BAT_VOLTAGE*2, temp ; ADC units
    CP16    bplus_filt,        temp
    bc      batmon_unload
    ; run discharge logic
    bra     batmon_discharger  ; & return


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Feed B+ values through filtering PID
;
; using a /256 error term & 2.5 msec poll, full-range transient response is ~2 seconds
;----------------------------------------------------------------------
;----------------------------------------------------------------------

; Entry conditions:  temp[4,5] = B+ raw A/D value
;
; Exit:              bplus_filt/err = updated

bat_input_PID ; 2.5 msec poll rate

    ; calc error term (err = current - previous)
    MOV16   temp+4,     temp+2  ; current V (ADC units)
    SUB16   bplus_filt, temp+2  ; temp[2] = error (10 bit) = current - previous

    ; add error to previous error remainder (cumulative error)
    ;  so filt_err = cumulative error term (x256)
    ;                low byte is fractional part
    ;                high byte is integer part
    ADD16   temp+2, bplus_filt_err  ; s15+s15

    ; add integer part of cumulative error to filtered value
    btfsc   bplus_filt_err+1, B7
    bra     batmon_PID_neg

    ;   remainder is positive
    ADD8to16 bplus_filt_err+1, bplus_filt
    clrf     bplus_filt_err+1   ; discard integer portion of positive remainder
    return

batmon_PID_neg
    ;   make cumulative error positive (necessary to extract integer part)
    NEG16   bplus_filt_err
    ;   apply error to filtered value
    SUB8from16 bplus_filt_err+1, bplus_filt
    ;   discard integer part of negative remainder
    clrf    bplus_filt_err+1
    NEG16   bplus_filt_err      ; restore remainder
    return


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Filtering PID for charge/load current (for display/logging purposes)
;----------------------------------------------------------------------
;----------------------------------------------------------------------

current_PID ; WREG = current setting value [POT units]
            ; updates current_filt, current_filt_err

    ; calc error
    clrf    temp+1
    movwf   temp+0
    movf    current_filt, w
    subwf   temp+0, w
    btfss   STATUS, C
    setf    temp+1
    ; add error to previous error remainder (cumulative error)
    addwf   current_filt_err+0, f
    movff   temp+1, WREG
    addwfc  current_filt_err+1, f

    ; add integer part of cumulative error to filtered value
    btfsc   current_filt_err+1, B7
    bra     current_PID_neg
    ;   remainder is positive
    movf    current_filt_err+1,w ; get integer portion
    addwf   current_filt, f
    btfsc   STATUS, C            ; rail at max value
    setf    current_filt
    clrf    current_filt_err+1   ; discard integer portion of positive remainder
    return

current_PID_neg
    ;   make cumulative error positive (necessary to extract integer part)
    NEG16   current_filt_err
    ;   apply error to filtered value
    movf    current_filt_err+1,w ; get integer portion
    subwf   current_filt, f
    btfss   STATUS, C            ; rail at min value
    clrf    current_filt
    ;   discard integer part of negative remainder
    clrf    current_filt_err+1
    NEG16   current_filt_err
    return


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Charger logic
;----------------------------------------------------------------------
;----------------------------------------------------------------------

;----------------------------------------------------------------------
; charge logic - poll at 2.5 msec rate
;
; Entry conditions:
;   battery context loaded
;   temp[4,5] = raw ADC B+ value

batmon_charger

    ; this routine applies only to CV mode
    btfsc   bat_flags, _BAT_CHARGE_CC ; CC is only used in diagnostic mode
    bra     batmon_set_pwr_notdead

    ; monitor for battery at full charge point
    ;
    ;    watch for bat_I == min (~50 mA) [for 5 seconds]

    ; but first, fetch vFull value
    MOV16   bat_vTarget, temp       ; Vx100
    RL16    temp, 1                 ; ADC units
    ; plus, termination condition does not apply if in diagnostic mode
    btfsc   bat_flags, _BAT_DIAGNOSTIC
    bra     batmon_chg_go
    ; ok, watch for continuous iChg ~ 0 [50 mA]
    movf    iChg_filt, w
    bnz     batmon_chg_reset
batmon_chg_dwell ; charge dwell timer handling
    movf    bat_timer, w            ; start timer if not started
    bnz     batmon_chg_dwell_running
    MOVLF   CHARGE_DWELL_TICKS, bat_timer
    bra     batmon_chg_go           ; continue for now
batmon_chg_dwell_running
    xorlw   0xFF                    ; 0xFF indicates that the timer expired
    bnz     batmon_chg_go           ; continue for now
    ; end of charge
batmon_chg_done
    bra     end_chg_or_load

batmon_chg_reset
    clrf    bat_timer               ; reset dwell timer

batmon_chg_go

    ; adjust chage current per B+
    ;
    ;   max charge current is C/2, and maximum current is 1 A
    ;
    ;   ALWAYS CV MODE (with limited max current)
    ;
    ;   for the PID, use raw ADC value as input [for loop stability]

    MOV16   temp+4, temp+2          ; B+ raw ADC (ADC units)
    SUB16   temp+2, temp            ; temp = error (10 bit) = Vfull - B+
    ;   tracking "PID" (not really, but has good noise immunity)
    movf    temp+0, w               ; zero error case
    iorwf   temp+1, w
    bz      batmon_set_power
    btfss   temp+1, B7              ; pos/neg error cases
    bra     batmon_chg_pos

    tstfsz  iChg_filt
    decf    iChg_filt, f
    bra     batmon_set_power

batmon_chg_pos
    incf    iChg_filt, w
    btfss   STATUS, Z
    incf    iChg_filt, f

batmon_set_power
    ; POT 255 = 1.05 A
    ; POT 0   = 0.05 A

    ; power limits do not apply when in diagnostic mode
    btfsc   bat_flags, _BAT_DIAGNOSTIC
    bra     batmon_set_pwr_notdead

    ; limit charge current to 1/2 C
    movf    iChg_limit, w       ; derived from C setting
    cpfslt  iChg_filt           ; don't raise current setting
    movwf   iChg_filt

    ; limit power if battery is "dead" ( << 3V )
    MOV16   bplus_filt,         temp ; ADC units (Vx2)
    SUBL16  DEAD_BAT_VOLTAGE*2, temp ; (C=0) if underflow (low voltage)
    bc      batmon_set_pwr_notdead
    movlw   POT_300MA
    cpfslt  iChg_filt           ;   don't raise current setting
    movwf   iChg_filt
batmon_set_pwr_notdead

    ; send POT setting to chip
    movf    iChg_filt, w
    call    digpot_set_power_1
    ; run current setting through results PID
    movf    iChg_filt, w
    bra     current_PID


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Discharge logic
;----------------------------------------------------------------------
;----------------------------------------------------------------------

;----------------------------------------------------------------------
; discharge logic - poll at 2.5 msec rate
;
; Entry conditions:
;   battery context loaded
;   temp[4,5] = raw ADC B+ value

batmon_discharger

    ; monitor for battery at discharge point
    ;
    ;   for MAX_POWER (CC) mode, watch for B+ < vEmpty [for 5 seconds]
    ;
    ;   for MAX_DISCH (CV) mode, watch for iLoad == 0 [for 5 seconds]

    ; but first, fetch target voltage value
    MOV16   bat_vTarget, temp       ; Vx100
    RL16    temp,1                  ; ADC units
    ; plus, termination condition does not apply if in diagnostic mode
    btfsc   bat_flags, _BAT_DIAGNOSTIC
    bra     batmon_disch_go
    ; ok, now handle by discharge mode
    btfss   bat_flags, _BAT_LOAD_CC
    bra     batmon_disch_chk_CV
    ; CC mode; watch for B+ staying below vEmpty
    CP16    bplus_filt, temp        ; ADC units
    bnc     batmon_disch_reset      ; B+ > vEmpty

batmon_disch_dwell ; discharge dwell timer handling
    movf    bat_timer, w            ; start timer if not started
    bnz     batmon_disch_dwell_running
    MOVLF   DISCH_DWELL_TICKS, bat_timer
    bra     batmon_disch_go         ; continue for now
batmon_disch_dwell_running
    xorlw   0xFF                    ; 0xFF indicates that the timer expired
    bnz     batmon_disch_go         ; continue for now
    ; end of discharge
batmon_disch_done
    bra     end_chg_or_load

batmon_disch_chk_CV
    ; CV mode; watch for continuous iLoad ~ 0
    movf    iLoad_filt, w
    bz      batmon_disch_dwell      ; min current is ~50 mA
batmon_disch_reset
    clrf    bat_timer               ; reset dwell timer

batmon_disch_go

    ; adjust dischage current per B+
    ;
    ;   max discharge current is C, and minimum C is 1 A-H, so max current is 1A
    ;
    ;   "MAX POWER" (CC) mode
    ;       run at max discharge current until vEmpty is seen for ~few seconds
    ;   "MAX DISCH" (CV) mode
    ;       PID B+ to POT such that B+ seeks vEmpty

    btfsc   bat_flags, _BAT_LOAD_CC ; CC mode current is constant
    bra     batmon_set_load

    ; "MAX DISCH" (CV) mode
    ;   for the load PID, use raw ADC value as input [for loop stability]
    MOV16   temp+4, temp+2      ; B+, raw, ADC units
    SUB16   temp,   temp+2      ; temp[2] = error (10 bit) = B+ - vEmpty
    ;   tracking "PID" (not really, but has good noise immunity)
    movf    temp+2, w           ; zero error case
    iorwf   temp+3, w
    bz      batmon_set_load
    btfss   temp+3, B7          ; pos/neg error cases
    bra     batmon_disch_pos

    tstfsz  iLoad_filt
    decf    iLoad_filt, f
    bra     batmon_set_load

batmon_disch_pos
    incf    iLoad_filt, w
    btfss   STATUS, Z
    incf    iLoad_filt, f

batmon_set_load
    ; limit maximum discharge current per setting
    movf    iLoad_limit, w      ; derived from batcfg_limit
    cpfslt  iLoad_filt          ; don't raise current setting
    movwf   iLoad_filt
    ; send POT setting to chip
    movf    iLoad_filt, w
    call    digpot_set_load_1
    ; run current setting through results PID
    movf    iLoad_filt, w
    bra     current_PID


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Analyzer Logic
;----------------------------------------------------------------------
;----------------------------------------------------------------------

; Public control functions for battery analyzer
;
; Entry conditions:
;     context_testtype   == desired analysis type
;     context_chargemenu == voltage selection

analyzer_activate   ; call from foregnd ONLY
                    ; this function may stall for up to ~200 usec

    call    reset_statistics        ; reset energy & time counters
    call    calc_bat_timeout        ; set operation timeout
    RL16    bat_timeout, 1          ;   x2 for analysis

    ; start 1'st analysis phase - precondition [always CV]
    MOVLF   ANALYZE_PRECONDITION,   bat_analyze
    ; start charge/discharge operation
    call    get_bplus_to_cnv_value  ; retrieve bplus[from interrupt] to cnv_value
    RR16    cnv_value, 1            ;   convert AD units to Vx100
    call    get_vFull               ; 2'nd parm = target voltage
    MOVLF   MASK(_BAT_CHARGE_ACTIVE), bat_request_data+0 ; 1'st parm = bat_flags
    setf    run_type
    CP16    cnv_value, bat_request_data+1 ; CY if bplus <= vTarget 
    bc      analyzer_activate_go
    MOVLF   MASK(_BAT_LOAD_ACTIVE), bat_request_data+0 ; 1'st parm = bat_flags
    clrf    run_type
analyzer_activate_go
    LED_OFF LED_GRN
    LED_ON  LED_RED
    bra     operation_request ; & return

;----------------------------------------------------------------------
; Call this routine when a charge|discharge completes.
;
; Exit:
;   bat_analyze = updated to next analysis state (or done)

end_chg_or_load
    SET_POPUP_MSG M_CHARGE_DONE
    rcall   batmon_operation_terminate

;batmon_analyzer

    tstfsz  bat_analyze
    bra     analyzer_is_on
    ; simple charge|discharge
    LED_OFF LED_RED
    LED_ON  LED_GRN
    return

analyzer_is_on

;-- analyzer is active

; analysis logic:
;    1) precondition - charge/discharge (to full)
;    2) discharge (to empty), measuring energy, in CC(power) or CV(energy) mode

    ; switch (bat_analyze)
    dcfsnz  bat_analyze, w
    bra     analyzer_state1 ; == ANALYZE_PRECONDITION
    dcfsnz  WREG, w
    bra     analyzer_state2 ; == ANALYZE_RUN
    clrf    bat_analyze
    return

analyzer_state1 ; precondition completed

    ; start measured discharge
    MOVLF   MASK(_BAT_LOAD_ACTIVE), bat_request_data+0 ; 1'st parm = bat_flags
    movff   context_testtype, WREG                     ;   CV=energy, CC=power
    btfss   WREG,B0                                    ;   choice 1   choice 0
    bsf     bat_request_data+0, _BAT_LOAD_CC
    rcall   get_vEmpty                                 ; 2'nd parm = target voltage
    MOV16   bat_request_data+1, run_vEnd               ; show the target [ending] voltage
    RL16    run_vEnd, 1                                ;   (V to ADC units)
    MOV16   bplus_filt, run_vStart                     ; show the starting voltage
    rcall   batmon_operation_request
    ;   for CC(max power) discharge, set load current to max
    movff   context_testtype, WREG                     ;   CV=energy, CC=power
    btfss   WREG,B0                                    ;   choice 1   choice 0
    setf    iLoad_filt
    ; bump state
    incf    bat_analyze, f
    return

analyzer_state2 ; discharge [measured] completed

    clrf    bat_analyze         ; "done" analyzer state
    LED_OFF LED_RED
    LED_ON  LED_GRN
    ; automatically show results [if on home page]
    movlw   HOME_PAGENUM
    cpfseq  page_current
    return  ; not on home page
    SET_BASE_PAGE RESULTS_RUN_PAGENUM
    return


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; Helpers
;----------------------------------------------------------------------
;----------------------------------------------------------------------

operation_request   ; call from foregnd ONLY
                    ; this function may stall for up to ~200 usec

    ; to avoid interrupt conflicts, must insure that:
    ;  1) the high interrupt is not active
    ;  2) the low interrupt won't start an ADC cycle while this function is executing
    bcf     INTCON, GIEL                ; insure low int won't start ADC cycle
operation_request_stall
      btfsc   pgm_flags, _FLAG_ADC      ; high interrupt active ?
      bra     operation_request_stall   ;   yes, stall
      rcall   batmon_operation_request  ;    no, ok to proceed
    bsf     INTCON, GIEL
    return

;- - - - - - - - - - - - - - - - - - - -

batmon_operation_request ; internal[this module|low interrupt] call only

    ; disallow if battery is not loaded [EXCEPT if diagnostic mode]
    btfsc   bat_request_data+0, _BAT_DIAGNOSTIC
    bra     boi_diag
    btfss   bat_flags, _BAT_LOADED
    return
boi_diag

    ; set operation bits of bat_flags
    movlw   MASK(_BAT_LOADED)
    andwf   bat_flags, f
    movf    bat_request_data+0, w
    andlw   0xFF ^ MASK(_BAT_LOADED)
    iorwf   bat_flags, f
    ; get target voltage
    MOV16   bat_request_data+1, bat_vTarget
    ; initialize test state machine
    clrf    bat_timer            ; dwell timer == off

    ; charger
    clrf    iChg_filt
    rcall   batmon_set_power    ; set charge current regulator to minimum
    btfsc   bat_flags, _BAT_CHARGE_ACTIVE
    rcall   batmon_power_on     ; turn on power switch 

    ; load
    clrf    iLoad_filt
    rcall   batmon_set_load     ; set load regulator to minimum
    btfsc   bat_flags, _BAT_LOAD_ACTIVE
    rcall   batmon_load_on      ; turn on power switch

    ; charge/load current for display/logging (not applicable in diagnostic modes)
    clrf    current_filt
    CLR     current_filt_err, 2

    return

;- - - - - - - - - - - - - - - - - - - -

operation_terminate ; foregnd entrypoint

batmon_operation_terminate

    ; record [actual] ending voltage for operation
    btfsc   bat_flags, _BAT_DIAGNOSTIC
    bra     bot_is_diag         ; does not apply to diagnostic modes
    MOV16   bplus_filt, run_vEnd
bot_is_diag

    ; turn off charger/discharger
    rcall   batmon_power_off    ; turn off charge power switch
    rcall   batmon_load_off     ; turn off load switch
    clrf    iChg_filt
    rcall   batmon_set_power    ; turn off power regulator (to min possible)
    clrf    iLoad_filt
    rcall   batmon_set_load     ; turn off load regulator (to min possible)

    ; set idle state
    movlw   MASK(_BAT_LOADED)
    andwf   bat_flags, f

    ; replot [CLS]  graph screen [to update mode text]
    ;   this is not really interrupt safe, but unlikely to cause a conflict
    movlw   RESULTS_GRAPH_PAGENUM
    cpfseq  page_current
    bra     bot_not_graph_page
    clrf    page_current    ; force screen clear
bot_not_graph_page

    return

;----------------------------------------------------------------------
; Target voltage helpers
;   called from foregnd & interrupt

get_vEmpty  ; to bat_request_data+1
    MOVLF16 STD_BAT_EMPTY,          bat_request_data+1 ; 2'nd parm = target voltage
    movff   context_chargemenu, WREG
    xorlw   3 ; "Custom" setting
    bnz     get_vEmpty_go
    MOV16   batcfg_vEmpty,          bat_request_data+1 ; 2'nd parm = target voltage
get_vEmpty_go
    return


get_vFull   ; to bat_request_data+1
    MOVLF16 STD_BAT_FULL, bat_request_data+1 ; default value
    movff   context_chargemenu, WREG
    movf    WREG,w
    bz      get_vFull_go
    decf    WREG,w
    bz      get_vFull_1
    decf    WREG,w
    bz      get_vFull_2
    decf    WREG,w
    bz      get_vFull_3
    return  ; invalid [program logic error]

get_vFull_1 ; "Military" charge settings
    MOVLF16 STD_BAT_MIL,   bat_request_data+1 ; 2'nd parm is target voltage
    bra     get_vFull_go

get_vFull_2 ; "Storage" charge settings
    MOVLF16 STD_BAT_STORE, bat_request_data+1 ; 2'nd parm is target voltage
    bra     get_vFull_go

get_vFull_3 ; "Custom" charge settings
    MOV16   batcfg_vFull,  bat_request_data+1 ; 2'nd parm is target voltage

get_vFull_go
    return

;----------------------------------------------------------------------
; Power/Load IO helpers

batmon_load_off
    bcf     LOAD_SWITCH_1   ; disconnect load
    return

batmon_load_on
    bsf     LOAD_SWITCH_1   ; turn om load
    return

batmon_power_off
    bcf     POWER_SWITCH_1  ; disconnect charge power
    return

batmon_power_on
    bsf     POWER_SWITCH_1  ; enable charge power
    return
