;**********************************************************************
;
;    I N T E R R U P T S
;
;    Allowed to use FSR2 only
;    Allowed to use sedondary SPI channel only
;    MUST preserve TBLPTR
;    NOT allowed to use Multiply
;    NOT allowed to access FLASH/EE [via EECON]
;
;**********************************************************************


;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;   HIGH PRIORITY     I N T E R R U P T     H A N D L E R
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------

; This interrupt stores the A/D channels sequentially into adc_data_aray.
;  AND calls the battery analyzer when there's a new battery voltage reading

HIGH_PRIORITY_INT

    ; we want to be able to use movff on WREG [in foregnd],
    ; but interrupts during such operations don't properly save WREG;
    ; so must use the following hack to preserve WREG across interrupted movff's
    call    foo,FAST        ; copy WREG to shadow
foo pop                     ; discard return to foo

;---------------------------------------------
;-- handle ADC interrupt
;---------------------------------------------

    ; reset interrupt source
    bcf     PIR1, ADIF

    ; store results in public location
    lfsr    2, adc_data_aray    ; located at beginning of bank
    rrncf   ADCON0, w           ; get selected channel (bits 5..2)
    andlw   b'00011110'
    movwf   FSR2L               ;   2x channel = word location
    movff   ADRESL, POSTINC2
    movff   ADRESH, INDF2

    ; last channel done ?
    movwf   int_temp+0          ; save this channel number (x2)
    xorlw   b'00001000'         ; last channel == 4
    bnz     ad_next
    ;   end A/D read sequence
    bcf     pgm_flags, _FLAG_ADC
    bra     ad_done

ad_next ; start read on next channel

    movlw   b'00000100'         ; channel field is bits 5..2
    addwf   ADCON0, f           ; select next channel
    bsf     ADCON0, GO          ; start A/D cycle (will generate interrupt when done)

ad_done ; process this A/D reading

    rrncf   int_temp+0, w       ; now have channel number (0,1,2,3,4)
    bz      ad_ch_BPLUS         ;  0 = B+
    decf    WREG,w
    bz      ad_exit             ;  1 = NTC[2] (handled in fgnd 1sec tick)
    decf    WREG,w
    bz      ad_exit             ;  2 = NTC[1] (handled in fgnd 1sec tick)
    decf    WREG,w
    bz      ad_exit             ;  3 = Vref (not used)
    decf    WREG,w
    bz      ad_exit             ;  4 = TPA (handled in 20msec tick)
    ; hmmm.
    bra     do_err_adc_sync

ad_ch_BPLUS ; B+
    rcall   batmon

ad_exit

;---------------------------------------------
;-- return from interrupt
;---------------------------------------------

    retfie  FAST    ; automatically restores W, STATUS, BSR regs

;- - - - - - - - - - - - - - - - - -

do_err_adc_sync     ; error handler helper [NOT REALLY SAFE FOR INTERRUPT USE]

    SET_ERROR_FREEZE E_ADC_SYNC
    bcf     pgm_flags, _FLAG_ADC
    bra     ad_exit


;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;   LOW PRIORITY     I N T E R R U P T     H A N D L E R
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------

;- - - - - - - - - - - - - - - - - -
; This interrupt is a "semi-precision" timer [crystal oscillator based]
;   (2.5 msec) for A/D read/PID
;   (20 msec) for human interface purposes

LOW_PRIORITY_INT

    ; preserve interrupted context
    movwf           int_save_W
    movff   STATUS, int_save_STATUS

;---------------------------------------------
;-- handle precision timebase interrupt
;---------------------------------------------

    ; reset interrupt source
    bcf     PIR1, CCP1IF

;-- 2.5 ms tick

    ; the ADC cycle SHOULD be complete by now
    btfsc   pgm_flags, _FLAG_ADC
    bra     do_err_adc_overrun      ; ADC cycle failed to complete

    ; initiate A/D read cycle
    btfsc   pgm_flags, _FLAG_ADC
    bra     do_err_adc_overrun
    MOVLF   b'00000001', ADCON0 ; select channel 0, ADC module ON
    bsf     ADCON0, GO          ; start A/D cycle (will generate interrupt when done)
    bsf     pgm_flags, _FLAG_ADC

    ; timer chain
    decfsz  TIMER_2p5MS, f
    bra     low_int_done
    MOVLF   _TIMER_2p5MS, TIMER_2p5MS

;-- 20 ms tick

    ; send tick to foregnd
    incf    foregnd_20ms_ticks,f ; let foregnd handle non-critical time tasks
    bnz     fgnd_20ms_ok
    ; foreground failed to process all the 20msec ticks (for ~2.5 seconds)
    ;  (by this time, however, the watchdog will have probably gone off)
    SET_ERROR_FREEZE E_20ms_OVERRUN
fgnd_20ms_ok

    ; watchdog interlock must be checked by foregnd mainloop
    bsf     diag_flags, _DIAG_WATCHDOG

    ; debug - blink diagnostic LED for 20 ms when triggered
    bsf     DIAG_LED
    btfsc   diag_flags, _DIAG_BLINK
    bcf     DIAG_LED
    bcf     diag_flags, _DIAG_BLINK

    ; monitor joystick input
    rcall   tpa_key_poll
    tstfsz  WREG
    movwf   tpa_key         ; pass to foregnd

;---------------------------------------------
;-- return from interrupt
;---------------------------------------------

low_int_done

    movf    int_save_W,      w
    movff   int_save_STATUS, STATUS

    retfie  ;not FAST; that's reserved for the high priority interrupt

;- - - - - - - - - - - - - - - - - -

do_err_adc_overrun  ; error handler helper [NOT REALLY SAFE FOR INTERRUPT USE]

    SET_ERROR_FREEZE E_ADC
    bra     low_int_done


;----------------------------------------------------------------------
;----------------------------------------------------------------------
; TPA Input Poll
;
; Debounce input signal, detect changes, return possible event to process
;----------------------------------------------------------------------
;----------------------------------------------------------------------

tpa_key_poll    ; poll at 20 mSec rate, returns key in WREG

    ; TPA debounce & convert to event
    ;      10bit 8bit actual  Key       KeyCode
    ; 5V = 1024  255   255    None      0
    ; 4V =  819  204   203    Left      24/44  0333  819
    ; 3V =  614  153   153    Down      23/43  026B  619
    ; 2V =  410  102   103    Right     22/42  01A1  417
    ; 1V =  205   51    49    Up        21/41  00C7  199
    ; 0V =    0    0     0    SEL/HOLD  15/16  0000

    ; retrieve TPA raw ADC value
    GET_INT_WORD adc_data_aray+2*ADC_DATAINDEX_TPA, tpa_raw
    ; only need 8 bit result
    rrcf    tpa_raw+1,f
    rrcf    tpa_raw+0,f
    rrcf    tpa_raw+1,f
    rrcf    tpa_raw+0,f     ; now it's 8 bit

    clrf    tpa_t0          ; TPA code 0 = Select
    movlw   11              ; first half-zone (0-10)
    subwf   tpa_raw, f
    bnc     tpa_key_found

tpa_key_loop
    incf    tpa_t0, f       ; bump key code
    movlw   5               ;   test for last key code
    xorwf   tpa_t0, w
    bz      tpa_key_found   ; TPA code 5 = no key
    ; span dead zone
    movlw   30              ; dead zone between valid key voltages
    subwf   tpa_raw, f
    bc      tpa_10
    MOVLF   TPA_BOUNCES, tpa_bounce ; reset debouncer countdown
    MOVLF   5, tpa_t0       ; assume no key
    retlw   KEY_NONE

tpa_10  ; span valid key zone
    movlw   21              ; valid key zone is +/- 10
    subwf   tpa_raw, f
    bc      tpa_key_loop

tpa_key_found
    movf    tpa_t1, w       ; get previous TPA code
    xorwf   tpa_t0, w       ; same as current TPA code?
    bz      tpa_key_steady  ;   yes
    MOVLF   TPA_BOUNCES, tpa_bounce ;    no - reset debouncer countdown
    movff   tpa_t0, tpa_t1          ;         reset TPA key state to follow TPA
    retlw   KEY_NONE

tpa_key_steady
    ; successive TPA values match
    movf    tpa_bounce, f   ; test for already debounced
    bnz     tpa_key_debounce
    ; generate hold-down key (one-shot)
    movf    tpa_hold, w     ; get hold timer
    bz      tpa_hold_end    ; already triggered or Off (== 0)?
    dcfsnz  tpa_hold, f     ;   no, count down hold time
    retlw   KEY_HOLD        ; key hold event triggered
tpa_hold_end
    ; generate continuous (repeating) keys
    decfsz  tpa_key_repeat, f   ; autorepeat rate divide-down from 50/sec
    retlw   KEY_NONE
    MOVLF   KEY_AUTOREPEAT_FAST, tpa_key_repeat
    ; return a repeating key
    movlw   5
    subwf   tpa_t0, w       ; expecting borrow (C=0) for keys 0..4
    btfsc   STATUS, C
    retlw   KEY_NONE
    movf    tpa_t0, w       ; get TPA key code (0..4)
    btfss   STATUS, Z
    iorlw   0x40            ; add repeat key code
    return

tpa_key_debounce
    decfsz  tpa_bounce, f   ; wait for debounce time to expire
    retlw   KEY_NONE
    ; TPA key event detected
    clrf    tpa_hold            ; turn off SELECT holddown timer
    MOVLF   KEY_AUTOREPEAT_START, tpa_key_repeat
    ; return non-repeating key
    movlw   5
    subwf   tpa_t0, w       ; expecting borrow (C=0) for keys 0..4
    btfsc   STATUS, C
    retlw   KEY_NONE
    movf    tpa_t0, w       ; get TPA key code (0..4)
    bz      TPA_cmd_select
    iorlw   0x20            ; add normal key code
    return

TPA_cmd_select      ; select pressed
    MOVLF   HOLDDOWN_TIME, tpa_hold
    retlw   KEY_SELECT
