;**********************************************************************
;
; LCD Page Display System
;
;**********************************************************************

; TO IMPLEMENT A PAGE...
;
;   1) Create the page script (pages.inc)
;      Note that the color palette is defined in the LCD library (LCD.inc)
;   2) Add the page to PAGE_LIST_TABLE (pages.inc)
;   3) Add a key handler [if needed] to foregnd.inc
;   4) For dynamic pages, add an entry to page_proc_table (foregnd.inc)
;      and create a page_update routine for the page.
;   5) Update the NBR_DYNAMIC_PAGES value (battery.asm) if adding a dynamic page
;
;   All pages must start with a 4-byte header:
;       1'st word = location [in flash] of the button handler for the page
;       2'nd word = bits that define which buttons are active
;       Note that popup pages NEVER receive any button events

TXT_END     EQU 0   ; required at end-of-page
TXT_AT      EQU 1   ; x position
TXT_COLOR   EQU 2   ; color constant (via palette)
TXT_BGND    EQU 3   ; color constant (via palette)
TXT_VCOLOR  EQU 4   ; color variable (indirect via palette)
TXT_FONT    EQU 5   ; font number
TXT_HLINE   EQU 6   ; draw a line (horizontal)
TXT_HDASH   EQU 7   ;  NOT IMPLEMENTED FOR PHILIPS CHIP
TXT_VLINE   EQU 8   ; draw a line (vertical)
TXT_VAR     EQU 9   ; variable data
TXT_CHAR    EQU 10  ; character constant
TXT_STRING  EQU 11  ; text stream
TXT_ERASE   EQU 12  ; block erase
TXT_IF      EQU 13  ; conditional (8 bit equality) branch
TXT_MODE    EQU 14  ; text transparent[default] vs. overstrike mode
TXT_PIXEL   EQU 15  ; individual pixel
TXT_IFBIT   EQU 16  ; conditional (bit set/clear) branch
TXT_GOTO    EQU 17  ; unconditional branch
TXT_CLS     EQU 18  ; force clear-screen (must be before "BACKGND" statement)
TXT_ARAY    EQU 19  ; text array
TXT_FCN     EQU 20  ; custom paint function call

FUNCTION macro target_label ; paint callback function
    dw  TXT_FCN, target_label
    endm

BRANCH macro target_label ; unconditional branch
    dw  TXT_GOTO, target_label
    endm

IFNE macro var, const, target_label ; 'var'[byte] in any memory bank
    dw  TXT_IF + ((const) << 8), var, target_label
    endm

IFBIT macro var, bit_number, bit_state, target_label ; 'var'[byte] in any memory bank
    dw  TXT_IFBIT, var, low(MASK(bit_number)) + ((bit_state*MASK(bit_number)) << 8), target_label
    endm

TEXT_OVERSTRIKE_MODE macro
    dw  TXT_MODE + (1 << 8)
    endm
TEXT_TRANSPARENT_MODE macro
    dw  TXT_MODE
    endm

AT macro x,y    ; set cursor position [origin is upper left corner of screen]
    dw  TXT_AT
    dw  x + ((129-(y)) << 8)
    endm

CLS macro       ; force screen clear
    dw  TXT_CLS
    endm

COLOR macro cv          ; color palette index (constant)
    dw  TXT_COLOR + (low(cv) << 8)
    endm

VCOLOR macro v          ; variable containing color palette index
    dw  TXT_VCOLOR + (low(v) << 8)
    endm

BACKGND macro cv     ; color palette index (constant) [REQUIRED at start of each page for CLS]
    dw  TXT_BGND  + (low(cv) << 8)
    endm

FONT macro f
    dw  TXT_FONT + (f << 8)
    endm

ERASE macro x,y,w,h
    dw  TXT_ERASE,  (130-(h)-(y)) + ((h) << 8), x + ((w) << 8)
    endm

CHAR macro ch
    dw  TXT_CHAR + ((ch) << 8)
    endm

; note that space characters should not be used in text!
; (any character value less than 33 is a dot spacing value)
TEXT macro w,s  ; string constant
    dw   TXT_STRING + ((w) << 8)
    data s
    endm

TEXT_ARAY macro v,a ; string array
    db  TXT_ARAY, 0
    dw  v   ; variable[byte] location, can be in any memory bank
    dw  a   ; text table location [in flash]
    endm

HLINE macro x,y,len ; [origin is lower left corner of screen]
    db  TXT_HLINE, y,x,len
    endm

HDASH macro x,y,len,style
    db  TXT_HDASH,  y,x,len, style
    endm

VLINE macro x,y,len ; [origin is lower left corner of screen]
    db  TXT_VLINE, x,y,len
    endm

PIXEL macro x,y     ; [origin is lower left corner of screen]
    db  TXT_PIXEL,x,y,0
    endm

VAR_1 macro v   ; single digit or character field
    dw  TXT_VAR + ((low(v)+0) << 8)
    endm

VAR_2 macro v   ; 2 char field
    dw  TXT_VAR + ((low(v)+0) << 8)
    dw  TXT_VAR + ((low(v)+1) << 8)
    endm

VAR_3 macro v   ; 3 char field
    dw  TXT_VAR + ((low(v)+0) << 8)
    dw  TXT_VAR + ((low(v)+1) << 8)
    dw  TXT_VAR + ((low(v)+2) << 8)
    endm

VAR_4 macro v   ; 5 char field
    dw  TXT_VAR + ((low(v)+0) << 8)
    dw  TXT_VAR + ((low(v)+1) << 8)
    dw  TXT_VAR + ((low(v)+2) << 8)
    dw  TXT_VAR + ((low(v)+3) << 8)
    endm

VAR_5 macro v   ; 5 char field
    dw  TXT_VAR + ((low(v)+0) << 8)
    dw  TXT_VAR + ((low(v)+1) << 8)
    dw  TXT_VAR + ((low(v)+2) << 8)
    dw  TXT_VAR + ((low(v)+3) << 8)
    dw  TXT_VAR + ((low(v)+4) << 8)
    endm

VAR_1D2 macro v ; #.## format field
    dw  TXT_VAR + ((low(v)+0) << 8)
    dw  TXT_CHAR + ('.' << 8)
    dw  TXT_VAR + ((low(v)+1) << 8)
    dw  TXT_VAR + ((low(v)+2) << 8)
    endm

VAR_5D2 macro v ; ###.## format field
    dw  TXT_VAR + ((low(v)+0) << 8)
    dw  TXT_VAR + ((low(v)+1) << 8)
    dw  TXT_VAR + ((low(v)+2) << 8)
    dw  TXT_CHAR + ('.' << 8)
    dw  TXT_VAR + ((low(v)+3) << 8)
    dw  TXT_VAR + ((low(v)+4) << 8)
    endm

VAR_5D3 macro v ; ##.### format field
    dw  TXT_VAR + ((low(v)+0) << 8)
    dw  TXT_VAR + ((low(v)+1) << 8)
    dw  TXT_CHAR + ('.' << 8)
    dw  TXT_VAR + ((low(v)+2) << 8)
    dw  TXT_VAR + ((low(v)+3) << 8)
    dw  TXT_VAR + ((low(v)+4) << 8)
    endm

VAR_5SD2 macro v ; +###.## format field
    dw  TXT_VAR + ((low(v)+0) << 8)
    dw  TXT_VAR + ((low(v)+1) << 8)
    dw  TXT_VAR + ((low(v)+2) << 8)
    dw  TXT_VAR + ((low(v)+3) << 8)
    dw  TXT_CHAR + ('.' << 8)
    dw  TXT_VAR + ((low(v)+4) << 8)
    dw  TXT_VAR + ((low(v)+5) << 8)
    endm

VAR_TXT macro n, v ; n char field (n < 16)
    local x = 0
    while x < n
    dw  TXT_VAR + ((low(v)+x) << 8)
x += 1
    endw
    endm

STOP macro
    dw  TXT_END
    endm

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; Splash Page
;

popup_splash

    dw      0,0 ; popups don't receive keys

    BACKGND PAL_splash_bgnd
    ; row 1
    FONT    FNT_24x10
    COLOR   PAL_splash_text
    AT      45,20
    TEXT     4,"LION"
    ; rows 2,3
    FONT    FNT_16x6
    AT      40,55
    TEXT    7,"BATTERY"
    AT      37,75
    TEXT    8,"ANALYZER"
    ; bottom row
show_copyright ; shared with About page
    FONT    FNT_16x6
    COLOR   PAL_copyright
    AT      5,112
    TEXT    17,"\x81\x022010\x06R.S.PIERCE"
    STOP

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; about page
;

about_page

    ; any key dismisses the About page
    dw      keyjumptab_about
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_LR)|MASK(KEYBIT_UD)|MASK(KEYBIT_REP)

    BACKGND PAL_main_bgnd
    ; title row
    FONT    FNT_24x10
    COLOR   PAL_title_text
    AT      35,5
    TEXT    5,"ABOUT"
    ; information area
    FONT    FNT_16x6
    COLOR   PAL_main_text
    AT      5,45
    TEXT    16,"BATTERY\x06ANALYZER"
    AT      12,70
    TEXT    14,"VERSION\x060931-A"
    ; bottom row
    BRANCH  show_copyright

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; Home page
;

home_page

    dw      keyjumptab_home
    dw      MASK(KEYBIT_LR)|MASK(KEYBIT_UD)|MASK(KEYBIT_SEL)

    BACKGND PAL_main_bgnd
    FONT    FNT_16x6

    COLOR   PAL_title_text
    AT      4,3
    TEXT    16,"BATTERY\x05ANALYZER"

    COLOR   PAL_main_text

    TEXT_OVERSTRIKE_MODE

    ; mode
    IFBIT   bat_flags, _BAT_CHARGE_ACTIVE, 1, home_is_chg
    IFBIT   bat_flags, _BAT_LOAD_ACTIVE,   1, home_is_dis
    AT      50,36
    TEXT    4,"IDLE"
    BRANCH  home_mode_done

home_is_chg
    AT      33,36
    TEXT    8,"CHARGING"
    BRANCH  home_mode_done

home_is_dis
    AT      22,36
    TEXT    11,"DISCHARGING"

home_mode_done

    HLINE   15,98,100
    HLINE   15,97,100

    HLINE   15,76,100
    HLINE   15,75,100

    AT      50,65
    VAR_1D2 display_V
    TEXT    2,"\x02V"

    IFBIT   bat_flags, _BAT_CHARGE_ACTIVE, 1, home_is_active
    IFBIT   bat_flags, _BAT_LOAD_ACTIVE,   1, home_is_active
    ERASE   37,85,7*6,16 ; blank current display field when idle
    STOP

home_is_active
    AT      37,85
    VAR_4   display_I
    TEXT    3,"\x02mA"

    STOP

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; Main Menu Page
;
;   About
;   Settings
;   Display
;   Exit

MENU_ITEM macro zY, zX, zIndex, zVariable, zLen, zTitle
    local not_selected, menu_rdy
    IFNE    zVariable, zIndex, not_selected
    COLOR   PAL_highlight
    BRANCH  menu_rdy
not_selected
    COLOR   PAL_main_text
menu_rdy
    AT      zX,zY
    TEXT    zLen,zTitle
    endm

;- - - - - - - - - - - - - - - - - - - -

main_page
    dw      keyjumptab_main
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_UD)|MASK(KEYBIT_LR)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      32,5
    TEXT    9,"MAIN\x05MENU"

    MENU_ITEM  29,40,0,context_mainmenu,7,"Battery"
    MENU_ITEM  48,40,1,context_mainmenu,7,"Display"
    MENU_ITEM  67,40,2,context_mainmenu,5,"Diags"
    MENU_ITEM  86,40,3,context_mainmenu,5,"About"
    MENU_ITEM 105,40,4,context_mainmenu,4,"Exit"

    STOP

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; Menu pages
;

battery_menu_page
    dw      keyjumptab_batmenu
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_UD)|MASK(KEYBIT_LR)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      36,5
    TEXT    7,"BATTERY"

    MENU_ITEM 35,40,0,context_batmenu,6,"Charge"
    MENU_ITEM 55,40,1,context_batmenu,7,"Analyze"
    MENU_ITEM 75,40,2,context_batmenu,7,"Results"
    MENU_ITEM 95,40,3,context_batmenu,4,"Exit"

    STOP

battery_interrupted_page
    dw      keyjumptab_batint
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_UD)|MASK(KEYBIT_LR)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      36,5
    TEXT    7,"BATTERY"

    MENU_ITEM 35,40,0,display_Index,4,"Stop"
    MENU_ITEM 55,40,1,display_Index,7,"Results"
    MENU_ITEM 75,40,2,display_Index,4,"Exit"

    STOP

batmenu_charge_page
    dw      keyjumptab_batcharge
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_UD)|MASK(KEYBIT_LR)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      36,5
    TEXT    7,"BATTERY"

    MENU_ITEM  31,40,0,context_chargemenu,8,"Standard"
    MENU_ITEM  51,40,1,context_chargemenu,8,"Military"
    MENU_ITEM  71,40,2,context_chargemenu,7,"Storage"
    MENU_ITEM  91,40,3,context_chargemenu,6,"Custom"
    MENU_ITEM 111,40,4,context_chargemenu,4,"Exit"

    STOP

batmenu_analyze_page
    dw      keyjumptab_batanalyze
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_UD)|MASK(KEYBIT_LR)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      36,5
    TEXT    7,"BATTERY"

    COLOR   PAL_main_text
    AT      20,30
    TEXT    13,"Analysis\x05Type"

    MENU_ITEM  53,33,0,context_testtype,10,"Power\x08(cc)"
    MENU_ITEM  73,33,1,context_testtype,11,"Energy\x03(cv)"
    MENU_ITEM  93,33,2,context_testtype, 4,"Exit"

    STOP

battery_capacity_page
    dw      keyjumptab_capacity
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_LR)|MASK(KEYBIT_UD)|MASK(KEYBIT_REP)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      6,3
    TEXT    16,"BATTERY\x05SETTINGS"

    FONT    FNT_24x10
    COLOR   PAL_settings_text
    AT      21,33
    TEXT    8,"CAPACITY"

    COLOR   PAL_highlight
    AT      30,70
    VAR_4   display_setting
    TEXT    3,"\x06MA"

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      15,110
    TEXT    14,"PRESS\x05TO\x05STORE"

    STOP

limit_text_table
    db      20, 4 ; elements, length
    data    "0.1C"
    data    "0.2C"
    data    "0.3C"
    data    "0.4C"
    data    "0.5C"
    data    "0.6C"
    data    "0.7C"
    data    "0.8C"
    data    "0.9C"
    data    "1.0C"
    data    "1.1C"
    data    "1.2C"
    data    "1.3C"
    data    "1.4C"
    data    "1.5C"
    data    "1.6C"
    data    "1.7C"
    data    "1.8C"
    data    "1.9C"
    data    "2.0C"

battery_limit_page
    dw      keyjumptab_limit
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_LR)|MASK(KEYBIT_UD)|MASK(KEYBIT_REP)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      6,3
    TEXT    16,"BATTERY\x05SETTINGS"

    FONT    FNT_24x10
    COLOR   PAL_settings_text
    AT      17,33
    TEXT    9,"DISCHARGE"

    COLOR   PAL_highlight
    AT      40,70
    TEXT_ARAY batcfg_limit, limit_text_table

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      15,110
    TEXT    14,"PRESS\x05TO\x05STORE"

    STOP

battery_Vfull_page

    dw      keyjumptab_Vfull
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_LR)|MASK(KEYBIT_UD)|MASK(KEYBIT_REP)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      6,3
    TEXT    16,"BATTERY\x05SETTINGS"

    FONT    FNT_24x10
    COLOR   PAL_settings_text
    AT      48,33
    TEXT    4,"FULL"

    COLOR   PAL_highlight
    AT      40,70
    VAR_1D2 display_setting
    TEXT    2, "\x06V"

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      15,110
    TEXT    14,"PRESS\x05TO\x05STORE"

    STOP

battery_Vempty_page
    dw      keyjumptab_Vempty
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_LR)|MASK(KEYBIT_UD)|MASK(KEYBIT_REP)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      6,3
    TEXT    16,"BATTERY\x05SETTINGS"

    FONT    FNT_24x10
    COLOR   PAL_settings_text
    AT      39,33
    TEXT    5,"EMPTY"

    COLOR   PAL_highlight
    AT      40,70
    VAR_1D2 display_setting
    TEXT    2, "\x06V"

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      15,110
    TEXT    14,"PRESS\x05TO\x05STORE"

    STOP

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 
; Results pages
;

;- - - - - - - - - - - - - - - - - - - -
;   Charged/Discharge
;   #.##V - #.##V
;   #### mAh
;   hh:mm:ss
results_page_run

    dw      keyjumptab_results_run
    dw      MASK(KEYBIT_LR)|MASK(KEYBIT_UD)|MASK(KEYBIT_SEL)

    BACKGND PAL_main_bgnd
    FONT    FNT_16x6

    COLOR   PAL_title_text
    IFNE    run_type, 0, results_is_load
    AT      40,7
    TEXT    6,"Charge"
    BRANCH  results_go
results_is_load
    AT      30,7
    TEXT    9,"Discharge"
results_go

    IFNE    run_time+0, 0, results_page_valid
    IFNE    run_time+1, 0, results_page_valid
    IFNE    run_time+2, 0, results_page_valid

    ; charge/discharge was not performed
    AT      15,30
    TEXT    13,"Not\x05Performed"
    STOP

results_page_valid
    COLOR   PAL_main_text

    ; charge/discharge was performed
    AT      25,30
    VAR_1D2 display_V0
    TEXT    5,"\x02V\x02-\x02"
    VAR_1D2 display_V1
    TEXT    2,"\x02V"

    TEXT_OVERSTRIKE_MODE

    AT      13,50
    VAR_4   display_I
    TEXT    3,"\x02mA"
    AT      75,50
    VAR_1D2 display_V
    TEXT    2,"\x02V"

    AT      39,70
    VAR_2   display_HH
    CHAR    ':'
    VAR_2   display_MM
    CHAR    ':'
    VAR_2   display_SS

    AT      35,90
    VAR_4   display_mAh
    TEXT    4,"\x02mAh"

    STOP

;- - - - - - - - - - - - - - - - - - - -
; graph page

results_page_graph

    dw      keyjumptab_results_graph
    dw      MASK(KEYBIT_LR)|MASK(KEYBIT_UD)|MASK(KEYBIT_SEL)

    BACKGND PAL_main_bgnd
    FONT    FNT_16x6

    ; mode
    COLOR   PAL_title_text
    IFBIT   bat_flags, _BAT_CHARGE_ACTIVE, 1, graph_is_chg
    IFBIT   bat_flags, _BAT_LOAD_ACTIVE,   1, graph_is_dis
    AT      48,2
    TEXT    4,"IDLE"
    BRANCH  graph_mode_done
graph_is_chg
    AT      33,2
    TEXT    8,"CHARGING"
    BRANCH  graph_mode_done
graph_is_dis
    AT      26,2
    TEXT    11,"DISCHARGING"
graph_mode_done

    COLOR   PAL_main_text
#if 0 ; screen coordinate test...
    HLINE   0,0,130
    VLINE   0,0,130
    HLINE   0,129,130
    VLINE   129,0,130
#endif
    VLINE   LOG_X_ZERO-1, LOG_Y_ZERO-1, LOG_DISPLAY_HEIGHT+1
    HLINE   LOG_X_ZERO-1, LOG_Y_ZERO-1, LOG_DISPLAY_WIDTH+1

    IFNE    bat_analyze, ANALYZE_PRECONDITION, graph_not_precondition
    AT      LOG_X_ZERO+11, 47
    TEXT    12,"conditioning"
    AT      LOG_X_ZERO+11, 65
    TEXT    12,"for\x05analysis"
graph_not_precondition

    COLOR   PAL_graph_V
    AT      8,44
    TEXT    1,"V"
    FUNCTION paint_results_V_graph

    COLOR   PAL_graph_I
    AT      9,62
    TEXT    1,"I"
    FUNCTION paint_results_I_graph

    TEXT_OVERSTRIKE_MODE

    COLOR   PAL_main_text
    AT      40,114
    VAR_2   display_HH
    CHAR    ':'
    VAR_2   display_MM
    CHAR    ':'
    VAR_2   display_SS

    STOP

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; 
; display settings pages
;

settings_menu_page

    dw      keyjumptab_settings
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_UD)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      38,5
    TEXT    7,"DISPLAY"

    MENU_ITEM 35,40,0,context_settings, 8,"Contrast"
    MENU_ITEM 55,40,1,context_settings,10,"Brightness"
    MENU_ITEM 75,40,2,context_settings, 4,"Exit"

    STOP

settings_backlight_page

    dw      keyjumptab_bklt
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_UD)|MASK(KEYBIT_REP)

    BACKGND PAL_main_bgnd
    COLOR   PAL_settings_text
    FONT    FNT_16x6

    AT      41,23
    TEXT    7,"DISPLAY"
    AT      34,42
    TEXT    9,"BACKLIGHT"

    FONT    FNT_24x10
    AT      40,69
    VAR_3   display_setting
    CHAR    '%'

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      15,110
    TEXT    14,"PRESS\x05TO\x05STORE"

    STOP

settings_contrast_page

    dw      keyjumptab_cont
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_UD)|MASK(KEYBIT_REP)

    BACKGND PAL_main_bgnd
    COLOR   PAL_settings_text
    FONT    FNT_16x6

    AT      41,23
    TEXT    7,"DISPLAY"
    AT      34,42
    TEXT    8,"CONTRAST"

    FONT    FNT_24x10
    AT      40,69
    VAR_3   display_setting
    CHAR    '%'

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      15,110
    TEXT    14,"PRESS\x05TO\x05STORE"

    STOP

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; diagnostics (self test / calibration)
;

diagnostics_menu_page
    dw      keyjumptab_diagmenu
    dw      MASK(KEYBIT_SEL)|MASK(KEYBIT_LR)|MASK(KEYBIT_UD)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text
    AT      24,5
    TEXT    11,"DIAGNOSTICS"

#define DIAGMENU_COL1 6
#define DIAGMENU_COL2 75

    MENU_ITEM  30,DIAGMENU_COL1,0,context_diagmenu,4,"Exit"
    MENU_ITEM  48,DIAGMENU_COL1,1,context_diagmenu,8,"LOAD\x03.1A"
    MENU_ITEM  66,DIAGMENU_COL1,2,context_diagmenu,8,"LOAD\x03.5A"
    MENU_ITEM  84,DIAGMENU_COL1,3,context_diagmenu,7,"LOAD\x041A"
    MENU_ITEM 102,DIAGMENU_COL1,4,context_diagmenu,7,"LOAD\x04CV"

    MENU_ITEM  30,DIAGMENU_COL2,5,context_diagmenu,8,"CHG\x03.05A"
    MENU_ITEM  48,DIAGMENU_COL2,6,context_diagmenu,7,"CHG\x03.1A"
    MENU_ITEM  66,DIAGMENU_COL2,7,context_diagmenu,7,"CHG\x03.5A"
    MENU_ITEM  84,DIAGMENU_COL2,8,context_diagmenu,6,"CHG\x041A"
    MENU_ITEM 102,DIAGMENU_COL2,9,context_diagmenu,6,"CHG\x04CV"

    STOP


diag_text_table
    db      9, 11 ; elements, length
    data    "LOAD\x06.1A\x00\x00\x00"
    data    "LOAD\x06.5A\x00\x00\x00"
    data    "LOAD\x061A\x00\x00\x00\x00"
    data    "LOAD\x06CV\x00\x00\x00\x00"
    data    "CHARGE\x06.05A"
    data    "CHARGE\x06.1A\x00"
    data    "CHARGE\x06.5A\x00"
    data    "CHARGE\x061A\x00\x00"
    data    "CHARGE\x06CV\x00\x00"

diagnostics_run_page
    dw      keyjumptab_runmenu
    dw      MASK(KEYBIT_SEL)

    BACKGND PAL_main_bgnd

    FONT    FNT_16x6
    COLOR   PAL_title_text

    COLOR   PAL_highlight
    AT      25,5
    TEXT_ARAY display_diag_setting, diag_text_table

    TEXT_OVERSTRIKE_MODE
    COLOR   PAL_main_text

    AT      5,30
    TEXT    7,"CHARGE\x06"
    VAR_4   display_diag_chg
    TEXT    3,"\x03mA"

    AT      5,48
    TEXT    5,"LOAD\x15"
    VAR_4   display_diag_load
    TEXT    3,"\x03mA"

    AT      5,70
    TEXT    7,"TARGET\x0A"
    IFNE    bat_vTarget+0, 0, diag_run_NNA
    IFNE    bat_vTarget+1, 0, diag_run_NNA
    TEXT    3,"N/A"
    BRANCH  diag_run_target_done
diag_run_NNA
    VAR_1D2 display_diag_target
    TEXT    2,"\x03V"
diag_run_target_done

    AT      5,88
    TEXT    7,"ACTUAL\x0A"
    VAR_1D2 display_diag_V
    TEXT    2,"\x03V"

    STOP

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; popup message page
;

popup_page  ; popup only
            ; REQUIRES "context_popup_msg" value

    dw      0,0 ; popups don't receive keys

    BACKGND PAL_main_bgnd
    COLOR   PAL_popup_text
    FONT    FNT_24x10

    IFNE context_popup_msg, M_INSERTED, popup_msg_not_0
    AT      29, 34
    TEXT    7,"BATTERY"
    AT      24, 70
    TEXT    8,"INSERTED"
    BRANCH  popup_msg_done

popup_msg_not_0
    IFNE context_popup_msg, M_REMOVED, popup_msg_not_1
    AT      29, 34
    TEXT    7,"BATTERY"
    AT      29, 70
    TEXT    7,"REMOVED"
    BRANCH  popup_msg_done

popup_msg_not_1
    IFNE context_popup_msg, M_CHARGING, popup_msg_not_2
    AT      32, 34
    TEXT    6,"CHARGE"
    AT      29, 70
    TEXT    7,"STARTED"
    BRANCH  popup_msg_done

popup_msg_not_2
    IFNE context_popup_msg, M_REMOVE_BAT, popup_msg_not_3
    AT      32, 34
    TEXT    6,"REMOVE"
    AT      26, 70
    TEXT    8,"BATTERY!"
    BRANCH  popup_msg_done

popup_msg_not_3
    IFNE context_popup_msg, M_CHARGE_DONE, popup_msg_not_4
    AT      32, 34
    TEXT    6,"CHARGE"
    AT      26, 70
    TEXT    8,"COMPLETE"
    BRANCH  popup_msg_done

popup_msg_not_4
    IFNE context_popup_msg, M_HEATSINK, popup_msg_not_5
    AT      32, 34
    TEXT    6,"SYSTEM"
    AT      26, 70
    TEXT    8,"OVERHEAT"
    BRANCH  popup_msg_done

popup_msg_not_5
    IFNE context_popup_msg, M_OVERHEAT, popup_msg_not_6
    AT      29, 34
    TEXT    7,"BATTERY"
    AT      26, 70
    TEXT    8,"OVERHEAT"
    BRANCH  popup_msg_done

popup_msg_not_6
    IFNE context_popup_msg, M_NORESULTS, popup_msg_not_7
    FONT    FNT_16x6
    AT      35,5
    TEXT     8,"Analysis"
    AT      23,25
    TEXT    12,"Conditioning"
    AT      26,45
    TEXT    11,"In\x05Progress"
    BRANCH  popup_msg_done

popup_msg_not_7
    IFNE context_popup_msg, M_NOBATTERY, popup_msg_not_8
    AT      29, 34
    TEXT    7,"BATTERY"
    AT      3, 70
    TEXT    11,"NOT\x07PRESENT"
    BRANCH  popup_msg_done

popup_msg_not_8
    IFNE context_popup_msg, M_ANALYZING, popup_msg_not_9
    AT      26, 34
    TEXT    8,"ANALYSIS"
    AT      29, 70
    TEXT    7,"STARTED"
    BRANCH  popup_msg_done

popup_msg_not_9
    AT      29, 34
    TEXT    7,"UNKNOWN"
    AT      29, 70
    TEXT    7,"MESSAGE"

popup_msg_done
    STOP

;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; error page
;

error_page  ; can be popup or "frozen" (base page)
            ; REQUIRES "context_error" value

    ; SEL key dismisses the error (frozen only)
    dw      keyjumptab_error
    dw      MASK(KEYBIT_SEL)

    BACKGND PAL_main_bgnd
    COLOR   PAL_popup_text
    FONT    FNT_24x10

    AT      30,5
    TEXT    6, "SYSTEM"
    AT      37,38
    TEXT    5, "ERROR"
    FONT    FNT_16x6

    IFNE context_error, E_20ms_OVERRUN, error_page_system_not_1
    AT      50,75
    TEXT    4,"TICK"
    AT      37,95
    TEXT    7,"OVERRUN"
    BRANCH  error_page_done
error_page_system_not_1

    IFNE context_error, E_DISPLAY, error_page_system_not_2
    AT      26,75
    TEXT    11,"BAD\x05DISPLAY"
    AT      50,95
    TEXT     4,"DATA"
    BRANCH  error_page_done
error_page_system_not_2

    IFNE context_error, E_ADC, error_page_system_not_3
    AT      25,75
    TEXT    9,"ADC\x05ERROR"
    AT      31,95
    TEXT     7,"OVERRUN"
    BRANCH  error_page_done
error_page_system_not_3

    IFNE context_error, E_ADC_SYNC, error_page_system_not_4
    AT      25,75
    TEXT    9,"ADC\x05ERROR"
    AT      27,95
    TEXT     8,"BAD\x05CHAN"
    BRANCH  error_page_done
error_page_system_not_4

    IFNE context_error, E_TIMEOUT, error_page_system_not_5
    AT      25,75
    TEXT    9,"OPERATION"
    AT      25,95
    TEXT     9,"TIMED\x05OUT"
    BRANCH  error_page_done
error_page_system_not_5

error_page_unknown
    FONT    FNT_16x6
    AT      36,75
    TEXT    7,"UNKNOWN"
    AT      46,95
    TEXT    5,"ERROR"
    BRANCH  error_page_done

error_page_done
    STOP


;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;
; Application Page Table
;
; Notes:
;   (1) The following table must align with the "do_page_update()" function (foregnd.inc)
;   (2) Sufficient dirty bit storage (page_dirty) must be allocated (1 byte per 8 dynamic pages)

PAGE_LIST_TABLE ; this table must align with page_proc_table( do_page_update in foregnd.inc )

   ;-- dynamic pages

   ; NOTE: MUST set "NBR_DYNAMIC_PAGES" (battery.asm) to the number of dynamic pages here

HOME_PAGENUM            EQU 0
    dw  home_page

MAIN_PAGENUM            EQU 1
    dw  main_page

SETTINGS_MENU_PAGENUM   EQU 2
    dw  settings_menu_page

SETTINGS_CONT_PAGENUM   EQU 3
    dw  settings_contrast_page 

SETTINGS_BKLT_PAGENUM   EQU 4
    dw  settings_backlight_page

BAT_CAPACITY_PAGE       EQU 5
    dw  battery_capacity_page

BAT_LIMIT_PAGE          EQU 6
    dw  battery_limit_page

BAT_VFULL_PAGE          EQU 7
    dw  battery_Vfull_page

BAT_VEMPTY_PAGE         EQU 8
    dw  battery_Vempty_page

BATTERY_INT_PAGENUM     EQU 9
    dw  battery_interrupted_page

BATTERY_MENU_PAGENUM    EQU 10
    dw  battery_menu_page

BATTERY_CHARGE_PAGENUM  EQU 11
    dw  batmenu_charge_page

BATTERY_ANALYZE_PAGENUM EQU 12
    dw  batmenu_analyze_page

DIAGMENU_PAGENUM        EQU 13
    dw  diagnostics_menu_page

DIAGRUN_PAGENUM         EQU 14
    dw  diagnostics_run_page

RESULTS_GRAPH_PAGENUM   EQU 15
    dw  results_page_graph

RESULTS_RUN_PAGENUM     EQU 16
    dw  results_page_run

#if RESULTS_RUN_PAGENUM != NBR_DYNAMIC_PAGES-1
 error "NBR_DYNAMIC_PAGES IS NOT SET CORRECTLY"
#endif

   ;-- static pages

SPLASH_PAGENUM          EQU NBR_DYNAMIC_PAGES+0
    dw  popup_splash         

ABOUT_PAGENUM           EQU NBR_DYNAMIC_PAGES+1
    dw  about_page            

ERROR_PAGENUM           EQU NBR_DYNAMIC_PAGES+2
    dw  error_page

POPUP_PAGENUM           EQU NBR_DYNAMIC_PAGES+3
    dw  popup_page
