;**********************************************************************
;
; System Initialization
;
;**********************************************************************

STARTUP ; power-on 

;-- entry conditions
;
;       The diag LED is on
;       I/O ports have been set up & initialized to default state

    INIT_PORTS  ; REinitialize IO state (in case of a soft reset from bootloader)

;-- zero all RAM (except Bootieloader's block at 0xF00)

    lfsr    0,0
zero_ram_loop               ; zero 0x000..0xEFF
    clrf    POSTINC0
    movlw   0x0F
    cpfseq  FSR0H
    bra zero_ram_loop

;-- A/D converter is used for various things

    ; Tad  = Fosc/32 = 1.25MHz = 0.8 usec
    ; Tacq = 20Tad = 16 usec
    ;
    ; Total conversion cycle time = (20+12)Tad = 26 usec

    MOVLF   b'10111010', ADCON2     ; right just (10 bit), TAQ=20Tad, TAD=Fosc/32 (max 40 MHz for 18F chip)
    MOVLF   b'00000001', ADCON0     ; Channel 0, A/D on
    bsf     PIE1, ADIE              ; enable A/D interrupt [high priority]
    bcf     PIR1, ADIF

;-- Timer 1 + CCP 1 is precision timebase (2.5 ms tick)

    ; Timer1 clock is fOsc/4 (10 MHz)
    ;   Tick rate is 2.5 msec
    ; 10 MHz => 2.5 msec == divide-by 25000

    MOVLF   b'10000001', T1CON      ; 16 bit mode, fOsc/4, prescale 1:1, timer off

    MOVLF   b'00001011', CCP1CON    ; compare mode: trigger special event (reset timer)
    MOVLF   high(25000), CCPR1H
    MOVLF    low(25000), CCPR1L
    bsf     PIE1, CCP1IE            ; tick interrupt is generated by CCP [low priority]
    bcf     IPR1, CCP1IP
    bcf     PIR1, CCP1IF

;--  Timer 2 + CCP 2 is LCD backlight PWM

    setf    PR2                     ; 9.765 kHz PWM frequency @ 10 mHz
    MOVLF   b'00000101', T2CON      ; postscale = 1:1, prescale = 1:4, timer = ON
    ; the backlight is never fully off because the LSB's are b'11'
    MOVLF   b'00111100', CCP2CON    ; duty cycle LSBs = b'11, mode = PWM
    clrf    CCPR2L                  ; duty cycle MSBs = 0 (100% OFF except for LSB's)
    ; do not enable backlight yet (wait until a CLS has been completed)

;-- SPI (to LCD)

    ; mixed use of bit-bang & SPI module (some bits are sent out manually)
    MOVLF   LCD_SSPCON1, LCD_CONST_SSPCON1
    MOVLF   LCD_SSPSTAT, LCD_CONST_SSPSTAT

;-- 100 ms delay

    MOVLF   17,2
poc_delay          
    decfsz  0,f
    bra poc_delay   ; 0.1 usec * 3 * 256 => 7.68 usec
    decfsz  1,f
    bra poc_delay   ; [7.68 usec + 0.1 usec ] * 3 * 256 => 5.975 msec
    decfsz  2,f
    bra poc_delay   ; 5.976 msec * 17 => 101 msec

;-- initialize variables

    ; initial state
    MOVLF   _TIMER_2p5MS, TIMER_2p5MS
    MOVLF   _TIMER_20MS,  TIMER_20MS
    MOVLF   _TIMER_100MS, TIMER_100MS
    MOVLF   AUTOREPLOT_TICKS, TIMER_autoreplot
    MOVLF   TPA_BOUNCES,  tpa_bounce
    setf    page_current

;-- load settings from EE

    ; display settings
    MOVLF   EEADR_DISPLAY, EEADR
    lfsr    0, LCD_backlight
    movlw   SIZEOF_DISPCFG
    call    EE_readblock
    bz      ee_display_load_ok
    ;   EE is invalid, reset it to defaults
    MOVLF   DEF_CONTRAST,    LCD_contrast
    MOVLF   DEF_BACKLIGHT,   LCD_backlight
ee_display_load_ok

    ; battery settings
    MOVLF   EEADR_BATTERY, EEADR
    lfsr    0, batcfg_C
    movlw   SIZEOF_BATCFG
    call    EE_readblock
    bz      ee_battery_load_ok
    ;   EE is invalid, reset it to defaults
    MOVLF16 DEF_BAT_C,      batcfg_C
    MOVLF16 DEF_BAT_FULL,   batcfg_vFull
    MOVLF16 DEF_BAT_EMPTY,  batcfg_vEmpty
    MOVLF   DEF_BAT_LIMIT,  batcfg_limit
ee_battery_load_ok

;-- set up initial display content

    ; normal startup; show splash message at power-on
    MOVLF   SPLASH_PAGENUM, page_temp
    MOVLF   SPLASH_PERSIST, page_persist
    ;   followed by initial display (after Splash)
    SET_BASE_PAGE  HOME_PAGENUM

;-- start WDT

    clrwdt
    bsf     WDTCON, SWDTEN  ; enable WDT

;-- enable interrupts

    bsf     RCON,   IPEN    ; enable interrupt priority levels
    bsf     INTCON, GIEH    ; high priority/global interrupt enable
    bsf     INTCON, GIEL    ; low priority/peripheral interrupt enable

    bsf     T1CON,  TMR1ON  ; start timebase

;-- initialize LCD subsystem

    call    LCD_palette_load
    call    LCD_initialize

;-- initialization complete, turn off diagnostic LED

    bsf     DIAG_LED

    goto    BOOT_RETURN ; return to bootloader
