;******************************************************************************************
; Microchip PIC18 Assembler Helper Macros          
;******************************************************************************************

;-- bit and mask keywords
#DEFINE B0 0 
#DEFINE B1 1
#DEFINE B2 2
#DEFINE B3 3
#DEFINE B4 4
#DEFINE B5 5
#DEFINE B6 6
#DEFINE B7 7
#define B8 8
#define B9 9
#define B10 10
#define B11 11
#define B12 12
#define B13 13
#define B14 14
#define B15 15 
#DEFINE MASK(b) (1 << b)

NOP_REPEAT macro zCount
    variable i
i=0
  while i < zCount
    nop
i=i+1
  endw
  endm

;- - - - - - - - - - - - - - - - - - - - - -
; Interrupt control
;- - - - - - - - - - - - - - - - - - - - - -

DISABLE_INTERRUPTS macro
    bcf     INTCON, GIEL
    bcf     INTCON, GIEH
  endm

ENABLE_INTERRUPTS macro
    bsf     INTCON, GIEH
    bsf     INTCON, GIEL
  endm

;- - - - - - - - - - - - - - - - - - - - - -
; Synthetic atomic memory access
;- - - - - - - - - - - - - - - - - - - - - -

GET_INT_WORD macro zFarAddr, zResult ; zFarAddr = any bank, zResult = access RAM
    local read_retry
read_retry ; read memory until consistent result is obtained
    MOV16   zFarAddr, zResult
    movff   zFarAddr+0, WREG
    cpfseq  zResult+0
    bra     read_retry
    movff   zFarAddr+1, WREG
    cpfseq  zResult+1
    bra     read_retry
    endm

;- - - - - - - - - - - - - - - - - - - - - -
; Generalized SPI transmit
;- - - - - - - - - - - - - - - - - - - - - -

SSP_WRITE_CONST macro zCONST ; destroys WREG
  local ssp_wait
    bcf     PIR1,   SSPIF
    MOVLF   zCONST, SSPBUF
ssp_wait
    btfss   PIR1,   SSPIF
    bra     ssp_wait
  endm

SSP_READ macro               ; does not destroy WREG
  local ssp_wait
    bcf     PIR1,   SSPIF
    movwf   SSPBUF
ssp_wait
    btfss   PIR1,   SSPIF
    bra     ssp_wait
  endm

SSP_WRITE_REG macro zREG     ; does not destroy WREG
  local ssp_wait
    bcf     PIR1,   SSPIF
    movff   zREG,   SSPBUF
ssp_wait
    btfss   PIR1,   SSPIF
    bra     ssp_wait
  endm


;- - - - - - - - - - - - - - - - - - - - - -
; Table Helpers
;- - - - - - - - - - - - - - - - - - - - - -

LOADTBLPTR macro zTable
    MOVLF    low(zTable), TBLPTRL
    MOVLF   high(zTable), TBLPTRH
  endm

CALC_TABLE_ADDRESS macro
    addwf   TBLPTRL, f
    clrf    WREG
    addwfc  TBLPTRH, f
  endm


;- - - - - - - - - - - - - - - - - - - - - -
; Initialize a register to a literal value
;- - - - - - - - - - - - - - - - - - - - - -

MOVLF macro zLITERAL, zREG  ; literal (8 bit)
;.fail zLITERAL ;debug - helps locate source line
    movlw   zLITERAL
    movwf   zREG
    endm

MOVLFB macro zLITERAL, zREG ; literal, banked
    movlw   zLITERAL
    movwf   zREG, BANKED
    endm

MOVLFF macro zLITERAL, zREG ; literal to far memory (without using banking)
    movlw   zLITERAL
    movff   WREG, zREG
    endm

MOVLF16 macro   zLITERAL, zREG  ; 16 bit literal
    movlw    low(zLITERAL)
    movwf   zREG+0
    movlw   high(zLITERAL)
    movwf   zREG+1
    endm

MOVLFF16 macro   zLITERAL, zREG  ; 16 bit literal
    MOVLFF   low(zLITERAL), zREG+0
    MOVLFF  high(zLITERAL), zREG+1
    endm

MOVLFB16 macro zLITERAL, zREG   ; 16 bit literal, banked
    movlw    low(zLITERAL)
    movwf   zREG+0, BANKED
    movlw   high(zLITERAL)
    movwf   zREG+1, BANKED
    endm


;- - - - - - - - - - - - - - - - - - - - - -
; Copy 16/24 bit value
;- - - - - - - - - - - - - - - - - - - - - -

MOV16 macro zSRC, zDST
    movff   zSRC+0, zDST+0
    movff   zSRC+1, zDST+1
    endm

MOV24 macro zSRC, zDST
    movff   zSRC+0, zDST+0
    movff   zSRC+1, zDST+1
    movff   zSRC+2, zDST+2
    endm


;- - - - - - - - - - - - - - - - - - - - - -
; Clear/Set a [small] number of bytes
;- - - - - - - - - - - - - - - - - - - - - -

CLR macro   zDST, zCount
    variable i
i=0
  while i < zCount
    clrf    zDST+i
i=i+1
  endw
    endm

CLRB macro   zDST, zCount
    variable i
i=0
  while i < zCount
    clrf    zDST+i, BANKED
i=i+1
  endw
    endm

SER macro   zDST, zCount
    variable i
i=0
  while i < zCount
    setf    zDST+i
i=i+1
  endw
    endm

SERB macro   zDST, zCount
    variable i
i=0
  while i < zCount
    setf    zDST+i, BANKED
i=i+1
  endw
    endm

;- - - - - - - - - - - - - - - - - - - - - -
; Signed arithmetic is soooo fun
;- - - - - - - - - - - - - - - - - - - - - -

NEG16 macro z16     ; negate a 15-bit signed number
    comf    z16+1,f
    negf    z16+0
    btfsc   STATUS, Z
    incf    z16+1,f
    endm

;- - - - - - - - - - - - - - - - - - - - - -
; Increment/Decrement
;- - - - - - - - - - - - - - - - - - - - - -

INC16 macro zDst
    incf    zDst+0, f   ; low byte
    btfsc   STATUS,Z
    incf    zDst+1, f   ; high byte
    endm

;- - - - - - - - - - - - - - - - - - - - - -
; Addition (DST = DST + SRC)
;   all of these destroy WREG
;- - - - - - - - - - - - - - - - - - - - - -
ADD16   macro   zSRC, zDST
    movf    zSRC+0, w    ; add low bytes
    addwf   zDST+0, f
    movff   zSRC+1, WREG ; doesn't affect C flag
    addwfc  zDST+1, f
    endm

ADD16B  macro   zSRC, zDST ; SRC = access RAM, DST = banked
    movf    zSRC+0, w    ; add low bytes
    addwf   zDST+0, f, BANKED
    movff   zSRC+1, WREG ; doesn't affect C flag
    addwfc  zDST+1, f, BANKED
    endm

ADD8to16 macro zSRC8, zDST16 ; SRC,DST in access RAM only
    movf    zSRC8, w     ; add low bytes
    addwf   zDST16+0, f
    clrf    WREG         ; doesn't affect C flag
    addwfc  zDST16+1, f
    endm

ADD8to24 macro zSRC8, zDST24 ; SRC,DST in access RAM only
    ADD8to16 zSRC8, zDST24
    addwfc  zDST24+2, f
    endm

ADD8to32 macro zSRC8, zDST32 ; SRC,DST in access RAM only
    ADD8to24 zSRC8, zDST32
    addwfc  zDST32+3, f
    endm

ADD16to32 macro zSRC16, zDST32 ; SRC,DST in access RAM only
    ADD16   zSRC16, zDST32
    clrf    WREG         ; doesn't affect C flag
    addwfc  zDST32+2,f
    addwfc  zDST32+3,f
    endm

ADDS16to24  macro   zSRC, zDST
    ADD16   zSRC, zDST
    clrf    WREG         ; doesn't affect C flag
    btfsc   zSRC+1, B7
    setf    WREG         ; extend sign
    addwfc  zDST+2, f
    endm

ADDS16to24B macro   zSRC, zDST ; SRC = access RAM, DST = banked
    ADD16B  zSRC, zDST
    clrf    WREG         ; doesn't affect C flag
    btfsc   zSRC+1, B7
    setf    WREG         ; extend sign
    addwfc  zDST+2, f, BANKED
    endm

ADDL16 macro zConst, zDst ; DST = DST + CONST
    movlw   low(zConst)
    addwf   zDst+0, f
    movlw   high(zConst)
    addwfc  zDst+1, f
    endm

;- - - - - - - - - - - - - - - - - - - - - -
; Subtraction (DST = DST - SRC)
; (CY) = 0 if underflow occurred
;   all of these destroy WREG
;- - - - - - - - - - - - - - - - - - - - - -

SUB16   macro   zSRC, zDST
    movf    zSRC+0, w    ; subtract low bytes
    subwf   zDST+0, f
    movff   zSRC+1, WREG ; doesn't affect C flag
    subwfb  zDST+1, f
    endm

SUB16B  macro   zSRC, zDST ; SRC = access RAM, DST = banked
    movf    zSRC+0, w    ; subtract low bytes
    subwf   zDST+0, f, BANKED
    movff   zSRC+1, WREG ; doesn't affect C flag
    subwfb  zDST+1, f, BANKED
    endm

SUB16F  macro   zSRC, zDST ; ARG1 = any, ARG2 = access RAM
    movff   zSRC+0, WREG ; subtract low bytes
    subwf   zDST+0, f
    movff   zSRC+1, WREG ; doesn't affect C flag
    subwfb  zDST+1, f
    endm

SUB8from16 macro zSRC8, zDST16 ; SRC,DST in access RAM only
    movf    zSRC8, w     ; subtract low bytes
    subwf   zDST16+0, f
    clrf    WREG         ; doesn't affect C flag
    subwfb  zDST16+1, f
    endm

SUB1624 macro   zSRC, zDST
    movf    zSRC+0, w    ; subtract low bytes
    subwf   zDST+0, f
    movff   zSRC+1, WREG ; doesn't affect C flag
    subwfb  zDST+1, f
    clrf    WREG         ; doesn't affect C flag
    subwfb  zDST+2, f
    endm

SUBL16 macro zConst, zDst ; DST = DST - CONST
    movlw   low(zConst)
    subwf   zDst+0, f
    movlw   high(zConst)
    subwfb  zDst+1, f
    endm

;- - - - - - - - - - - - - - - - - - - - - -
; 16 bit compare 
; (CY) = 0 if (ARG1 > ARG2)
;- - - - - - - - - - - - - - - - - - - - - -

CP16    macro   zARG1, zARG2
    movf    zARG1+0, w
    subwf   zARG2+0, w
    movff   zARG1+1, WREG
    subwfb  zARG2+1, w
    endm

CP16F   macro   zARG1, zARG2 ; ARG1 = any, ARG2 = access RAM
    movff   zARG1+0, WREG
    subwf   zARG2+0, w
    movff   zARG1+1, WREG
    subwfb  zARG2+1, w
    endm

#if 0 ; this is wrong...
CP16K   macro   zARG1, zARG2 ; ARG2 is a constant
    movf    zARG1+0, w
    sublw   low(zARG2)
    movff   zARG1+1, WREG
    subwfb  high(zARG2), w  ;NOT!
    endm
#endif

;- - - - - - - - - - - - - - - - - - - - - -
; bit rotate
;- - - - - - - - - - - - - - - - - - - - - -

RR16 macro zArg, zBits
    variable i
i=0
  while i < zBits
    rrcf    zArg+1,f
    rrcf    zArg+0,f
    bcf     zArg+1,B7
i=i+1
  endw
    endm

RR16S macro zWord, zBits ; signed 16-bit shift
    variable i
i=0
  while i < zBits
    rlcf    zWord+1,w   ; extend sign (copy MSB to CY)
    rrcf    zWord+1,f
    rrcf    zWord+0,f
i=i+1
  endw
    endm

RL16 macro zWord, zBits
    variable i
i=0
  while i < zBits
    rlcf    zWord+0,f
    rlcf    zWord+1,f
    bcf     zWord+0,B0
i=i+1
  endw
    endm
